@@ -42,6 +42,7 @@
 #define CMAN_CMD_SET_DIRTY          0x800000c2
 #define CMAN_CMD_SET_DEBUGLOG       0x800000c3
 #define CMAN_CMD_DUMP_OBJDB         0x800000c4
+#define CMAN_CMD_GETNODE_EXTRA      0x000000c5
 
 #define CMAN_CMD_DATA               0x00000100
 #define CMAN_CMD_BIND               0x00000101
@@ -162,6 +163,7 @@ struct sock_confchg_message {
 #define CMAN_EXTRA_FLAG_SHUTDOWN  4
 #define CMAN_EXTRA_FLAG_UNCOUNTED 8
 #define CMAN_EXTRA_FLAG_DIRTY    16
+#define CMAN_EXTRA_FLAG_DISALLOWED_ENABLED    32
 
 struct cl_extra_info {
 	int           node_state;
@@ -193,6 +195,16 @@ struct cl_cluster_node {
 	unsigned char votes;
 };
 
+struct cl_node_extra
+{
+	int nodeid;
+	int state;
+	int votes;
+	int expected_votes;
+	int leave_reason;
+};
+
+
 /* Structure passed to CMAN_CMD_ISLISTENING */
 struct cl_listen_request {
 	unsigned char port;
