@@ -13,6 +13,7 @@
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
+#include <dirent.h>
 #include <linux/major.h>
 #ifdef HAVE_SELINUX
 #include <selinux/selinux.h>
@@ -23,11 +24,11 @@
 #include "libdlm.h"
 #include <linux/dlm_device.h>
 
-#define PROC_MISC		"/proc/misc"
 #define MISC_PREFIX		"/dev/misc/"
 #define DLM_PREFIX		"dlm_"
 #define DLM_MISC_PREFIX		MISC_PREFIX DLM_PREFIX
-#define DLM_CONTROL_DEV		"dlm-control"
+#define DLM_CONTROL_NAME	"dlm-control"
+#define DLM_CONTROL_PATH	MISC_PREFIX DLM_CONTROL_NAME
 #define DEFAULT_LOCKSPACE	"default"
 
 /*
@@ -73,11 +74,6 @@ struct dlm_lock_result_v5 {
 	__u32 lvb_offset;
 };
 
-/*
- * This is the name of the control device
- */
-
-#define DLM_CTL_DEVICE_NAME MISC_PREFIX DLM_CONTROL_DEV
 
 /*
  * One of these per lockspace in use by the application
@@ -113,28 +109,6 @@ static void ls_dev_name(const char *lsname, char *devname, int devlen)
 	snprintf(devname, devlen, DLM_MISC_PREFIX "%s", lsname);
 }
 
-#ifdef HAVE_SELINUX
-static int set_selinux_context(const char *path)
-{
-	security_context_t scontext;
-
-	if (is_selinux_enabled() <= 0)
-		return 1;
-
-	if (matchpathcon(path, 0, &scontext) < 0) {
-		return 0;
-	}
-
-	if ((lsetfilecon(path, scontext) < 0) && (errno != ENOTSUP)) {
-		freecon(scontext);
-		return 0;
-	}
-
-	free(scontext);
-	return 1;
-}
-#endif
-
 static void dummy_ast_routine(void *arg)
 {
 }
@@ -270,7 +244,7 @@ static int ls_pthread_cleanup(struct dlm_ls_info *lsinfo)
 }
 
 /* Cleanup default lockspace */
-int dlm_pthread_cleanup()
+int dlm_pthread_cleanup(void)
 {
     struct dlm_ls_info *lsinfo = default_ls;
 
@@ -337,86 +311,6 @@ static int open_default_lockspace(void)
 	return 0;
 }
 
-static int create_control_device(void)
-{
-    FILE *pmisc;
-    int minor;
-    char name[256];
-    int status = 0;
-    int saved_errno = 0;
-    mode_t oldmode;
-    int done = 0;
-    int rv;
-
-    /* Make sure the parent directory exists */
-    oldmode = umask(0);
-    status = mkdir(MISC_PREFIX, 0755);
-    umask(oldmode);
-    if (status != 0 && errno != EEXIST)
-    {
-	return status;
-    }
-
-    pmisc = fopen(PROC_MISC, "r");
-    if (!pmisc)
-	return -1;
-
-    while (!feof(pmisc))
-    {
-	
-	rv = fscanf(pmisc, "%d %s\n", &minor, name);
-	if ((rv == EOF) || (rv != 2))
-		break;
-	if (strcmp(name, DLM_CONTROL_DEV) == 0)
-	{
-	    status = mknod(DLM_CTL_DEVICE_NAME, S_IFCHR | 0600, makedev(MISC_MAJOR, minor));
-	    saved_errno = errno;
-	    done = 1;
-#ifdef HAVE_SELINUX
-	    if (status == 0)
-		set_selinux_context(DLM_CTL_DEVICE_NAME);
-#endif
-	    break;
-	}
-    }
-    fclose(pmisc);
-
-    /* if it all went well but we didn't find the DLM misc device, still return an error */
-    if (status == 0 && !done)
-    {
-	    status = -1;
-	    saved_errno = ENXIO;
-    }
-    errno = saved_errno;
-    return status;
-}
-
-static int find_minor_from_proc(const char *prefix, const char *lsname)
-{
-    FILE *f = fopen(PROC_MISC, "r");
-    char miscname[strlen(lsname)+strlen(prefix)+1];
-    char name[256];
-    int minor;
-
-    sprintf(miscname, "%s%s", prefix, lsname);
-
-    if (f)
-    {
-	while (!feof(f))
-	{
-	    if (fscanf(f, "%d %s", &minor, name) == 2 &&
-		strcmp(name, miscname) == 0)
-	    {
-		fclose(f);
-		return minor;
-	    }
-	}
-    }
-
-    fclose(f);
-    return 0;
-}
-
 static void detect_kernel_version(void)
 {
 	struct dlm_device_version v;
@@ -436,32 +330,65 @@ static void detect_kernel_version(void)
 	kernel_version_detected = 1;
 }
 
+static int find_control_minor(int *minor)
+{
+	FILE *f;
+	char name[256];
+	int found = 0, m = 0;
+
+	f = fopen("/proc/misc", "r");
+	if (!f)
+		return -1;
+
+	while (!feof(f)) {
+		if (fscanf(f, "%d %s", &m, name) != 2)
+			continue;
+		if (strcmp(name, DLM_CONTROL_NAME))
+			continue;
+		found = 1;
+		break;
+	}
+	fclose(f);
+
+	if (found) {
+		*minor = m;
+		return 0;
+	}
+	return -1;
+}
+
 static int open_control_device(void)
 {
-	int minor;
 	struct stat st;
-	int stat_ret;
+	int i, rv, minor, found = 0;
 
-	if (control_fd == -1) {
-		stat_ret = stat(DLM_CTL_DEVICE_NAME, &st);
-		if (!stat_ret) {
-			minor = find_minor_from_proc("", DLM_CONTROL_DEV);
-			if (S_ISCHR(st.st_mode) &&
-			    st.st_rdev != makedev(MISC_MAJOR, minor))
-				unlink(DLM_CTL_DEVICE_NAME);
-		}
+	if (control_fd > -1)
+		goto out;
 
-		control_fd = open(DLM_CTL_DEVICE_NAME, O_RDWR);
+	rv = find_control_minor(&minor);
+	if (rv < 0)
+		return -1;
 
-		if (control_fd == -1) {
-			if (create_control_device())
-				return -1;
+	/* wait for udev to create the device */
 
-			control_fd = open(DLM_CTL_DEVICE_NAME, O_RDWR);
-			if (control_fd == -1)
-				return -1;
+	for (i = 0; i < 10; i++) {
+		if (stat(DLM_CONTROL_PATH, &st) == 0 &&
+		    minor(st.st_rdev) == minor) {
+			found = 1;
+			break;
 		}
+		sleep(1);
+		continue;
 	}
+
+	if (!found)
+		return -1;
+
+	control_fd = open(DLM_CONTROL_PATH, O_RDWR);
+	if (control_fd == -1)
+		return -1;
+
+ out:
 	fcntl(control_fd, F_SETFD, 1);
 
 	if (!kernel_version_detected)
@@ -469,6 +396,64 @@ static int open_control_device(void)
 	return 0;
 }
 
+/* the max number of characters in a sysfs device name, not including \0 */
+#define MAX_SYSFS_NAME 19
+
+static int find_udev_device(const char *lockspace, int minor, char *udev_path)
+{
+	char basename[PATH_MAX];
+	char tmp_path[PATH_MAX];
+	DIR *d;
+	struct dirent *de;
+	struct stat st;
+	size_t basename_len;
+	int i;
+
+	ls_dev_name(lockspace, udev_path, PATH_MAX);
+	snprintf(basename, PATH_MAX, DLM_PREFIX "%s", lockspace);
+	basename_len = strlen(basename);
+
+	for (i = 0; i < 10; i++) {
+
+		/* look for a device with the full name */
+
+		if (stat(udev_path, &st) == 0 && minor(st.st_rdev) == minor)
+			return 0;
+
+		if (basename_len < MAX_SYSFS_NAME) {
+			sleep(1);
+			continue;
+		}
+
+		/* look for a device with a truncated name */
+
+		d = opendir(MISC_PREFIX);
+		while ((de = readdir(d))) {
+			if (de->d_name[0] == '.')
+				continue;
+			if (strlen(de->d_name) < MAX_SYSFS_NAME)
+				continue;
+			if (strncmp(de->d_name, basename, MAX_SYSFS_NAME))
+				continue;
+			snprintf(tmp_path, PATH_MAX, MISC_PREFIX "%s",
+				 de->d_name);
+			if (stat(tmp_path, &st))
+				continue;
+			if (minor(st.st_rdev) != minor)
+				continue;
+
+			/* truncated name */
+			strncpy(udev_path, tmp_path, PATH_MAX);
+			closedir(d);
+			return 0;
+		}
+		closedir(d);
+		sleep(1);
+	}
+
+	return -1;
+}
+
 /*
  * do_dlm_dispatch()
  * Read an ast from the kernel.
@@ -1189,14 +1174,16 @@ int dlm_ls_get_fd(dlm_lshandle_t lockspace)
 #ifdef _REENTRANT
 static void *dlm_recv_thread(void *lsinfo)
 {
-    struct dlm_ls_info *lsi = lsinfo;
+	struct dlm_ls_info *lsi = lsinfo;
 
-    for (;;)
-	do_dlm_dispatch(lsi->fd);
+	for (;;)
+		do_dlm_dispatch(lsi->fd);
+
+	return NULL;
 }
 
 /* Multi-threaded callers normally use this */
-int dlm_pthread_init()
+int dlm_pthread_init(void)
 {
     if (open_default_lockspace())
 	return -1;
@@ -1290,10 +1277,10 @@ static int create_lockspace_v6(const char *name, uint32_t flags)
 static dlm_lshandle_t create_lockspace(const char *name, mode_t mode,
 				       uint32_t flags)
 {
-	int minor, i;
-	struct stat st;
-	char dev_name[PATH_MAX];
+	char dev_path[PATH_MAX];
+	char udev_path[PATH_MAX];
 	struct dlm_ls_info *newls;
+	int error, saved_errno, minor;
 
 	/* We use the control device for creating lockspaces. */
 	if (open_control_device())
@@ -1303,39 +1290,48 @@ static dlm_lshandle_t create_lockspace(const char *name, mode_t mode,
 	if (!newls)
 		return NULL;
 
-	ls_dev_name(name, dev_name, sizeof(dev_name));
+	ls_dev_name(name, dev_path, sizeof(dev_path));
 
 	if (kernel_version.version[0] == 5)
 		minor = create_lockspace_v5(name, flags);
 	else
 		minor = create_lockspace_v6(name, flags);
 
-	if (minor < 0) {
-		free(newls);
-		return NULL;
-	}
+	if (minor < 0)
+		goto fail;
 
-	/* Wait for udev to create the device */
-	for (i = 1; i < 10; i++) {
-		if (stat(dev_name, &st) == 0)
-			break;
-		sleep(1);
+	/* Wait for udev to create the device; the device it creates may
+	   have a truncated name due to the sysfs device name limit. */
+	   
+	error = find_udev_device(name, minor, udev_path);
+	if (error)
+		goto fail;
+
+	/* If the symlink already exists, find_udev_device() will return
+	   it and we'll skip this. */
+
+	if (strcmp(dev_path, udev_path)) {
+		error = symlink(udev_path, dev_path);
+		if (error)
+			goto fail;
 	}
 
 	/* Open it and return the struct as a handle */
 
-	newls->fd = open(dev_name, O_RDWR);
-	if (newls->fd == -1) {
-		int saved_errno = errno;
-		free(newls);
-		errno = saved_errno;
-		return NULL;
-	}
+	newls->fd = open(dev_path, O_RDWR);
+	if (newls->fd == -1)
+		goto fail;
 	if (mode)
 		fchmod(newls->fd, mode);
 	newls->tid = 0;
 	fcntl(newls->fd, F_SETFD, 1);
 	return (dlm_lshandle_t)newls;
+
+ fail:
+	saved_errno = errno;
+	free(newls);
+	errno = saved_errno;
+	return NULL;
 }
 
 dlm_lshandle_t dlm_new_lockspace(const char *name, mode_t mode, uint32_t flags)
@@ -1382,9 +1378,15 @@ static int release_lockspace(uint32_t minor, uint32_t flags)
 
 int dlm_release_lockspace(const char *name, dlm_lshandle_t ls, int force)
 {
+	char dev_path[PATH_MAX];
 	struct stat st;
 	struct dlm_ls_info *lsinfo = (struct dlm_ls_info *)ls;
 	uint32_t flags = 0;
+	int fd, is_symlink = 0;
+
+	ls_dev_name(name, dev_path, sizeof(dev_path));
+	if (!lstat(dev_path, &st) && S_ISLNK(st.st_mode))
+		is_symlink = 1;
 
 	/* We need the minor number */
 	if (fstat(lsinfo->fd, &st))
@@ -1400,6 +1402,26 @@ int dlm_release_lockspace(const char *name, dlm_lshandle_t ls, int force)
 		flags = DLM_USER_LSFLG_FORCEFREE;
 
 	release_lockspace(minor(st.st_rdev), flags);
+
+	if (!is_symlink)
+		return 0;
+
+	/* The following open is used to detect if our release was the last.
+	   It will fail if our release was the last, because either:
+	   . udev has already removed the truncated sysfs device name (ENOENT)
+	   . the misc device has been deregistered in the kernel (ENODEV)
+	     (the deregister completes before release returns)
+
+	   So, if the open fails, we know that our release was the last,
+	   udev will be removing the device with the truncated name (if it
+	   hasn't already), and we should remove the symlink. */
+
+	fd = open(dev_path, O_RDWR);
+	if (fd < 0)
+		unlink(dev_path);
+	else
+		close(fd); /* our release was not the last */
+
 	return 0;
 }
 
