@@ -4,8 +4,12 @@
 #ifdef __linux__
 #include <pthread.h>
 #include <errno.h>
+#include <string.h>
+#include <stdlib.h>
+#include <stdio.h>
 #include <sys/types.h>
 #include <netdb.h>
+#include <unistd.h>
 
 #include <libdlm.h>
 
@@ -55,7 +59,7 @@ static int *lvb_int = (int *)&our_lksb.sb_lvb;
 #define SUCCESS SS$_NORMAL
 #endif
 
-static char *lockname="ping";
+static const char *lockname="ping";
 static int us = 1;
 static int maxnode = 2;
 static int cur_mode;
@@ -67,7 +71,6 @@ static void blockast_routine(void *arg);
 #ifdef __linux__
 static int convert_lock(int mode)
 {
-    int old_mode = cur_mode;
     int status;
 
     printf("pinglock: convert to %d starting\n", mode);
@@ -93,7 +96,7 @@ static int convert_lock(int mode)
     return status;
 }
 
-static void unlock()
+static void unlock(void)
 {
     int status;
     status = dlm_unlock( our_lksb.sb_lkid,
@@ -105,7 +108,7 @@ static void unlock()
 
 }
 
-static void start_lock()
+static void start_lock(void)
 {
     int status;
     cur_mode = LKM_EXMODE;
