@@ -169,10 +169,12 @@ def main():
 		       "test", "port", "separator", "no_login", "no_password",
 		       "snmp_version", "community", "snmp_auth_prot", "snmp_sec_level",
 		       "snmp_priv_prot", "snmp_priv_passwd", "snmp_priv_passwd_script",
-		       "udpport","inet4_only","inet6_only"]
+		       "udpport","inet4_only","inet6_only",
+		       "power_timeout", "shell_timeout", "login_timeout", "power_wait" ]
 
 	atexit.register(atexit_handler)
 
+	snmp_define_defaults ()
 	apc_snmp_define_defaults()
 
 	options=check_input(device_opt,process_input(device_opt))
@@ -187,8 +189,17 @@ def main():
 	if (not (options.has_key("-s"))):
 		options["-s"]="1"
 
-	# Operate the fencing device
-	fence_action(FencingSnmp(options), options, set_power_status, get_power_status, get_outlets_status)
+	docs = { }
+	docs["shortdesc"] = "Fence agent for APC over SNMP"
+	docs["longdesc"] = "fence_apc_snmp is an I/O Fencing agent \
+which can be used with the APC network power switch. It logs \
+into a device via SNMP and reboots a specified outlet. It supports \
+SNMP v1 and v3 with all combinations of  authenticity/privacy settings."
+	show_docs(options, docs)
 
+	# Operate the fencing device
+	result = fence_action(FencingSnmp(options), options, set_power_status, get_power_status, get_outlets_status)
+
+	sys.exit(result)
 if __name__ == "__main__":
 	main()
