@@ -1,12 +1,14 @@
 #include <linux/sched.h>
+#include <linux/fs_struct.h>
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
 #include <linux/spinlock.h>
-#include <asm/semaphore.h>
+#include <linux/semaphore.h>
 #include <linux/completion.h>
 #include <linux/buffer_head.h>
 #include <linux/posix_acl.h>
 #include <linux/posix_acl_xattr.h>
+#include <linux/cred.h>
 
 #include "gfs.h"
 #include "acl.h"
@@ -78,7 +80,7 @@ gfs_acl_validate_remove(struct gfs_inode *ip, int access)
 {
 	if (!ip->i_sbd->sd_args.ar_posix_acls)
 		return -EOPNOTSUPP;
-	if (current->fsuid != ip->i_di.di_uid && !capable(CAP_FOWNER))
+	if (current_fsuid() != ip->i_di.di_uid && !capable(CAP_FOWNER))
 		return -EPERM;
 	if (ip->i_di.di_type == GFS_FILE_LNK)
 		return -EOPNOTSUPP;
@@ -125,7 +127,7 @@ gfs_acl_get(struct gfs_inode *ip, int access, struct posix_acl **acl)
 	if (!GFS_EA_DATA_LEN(el.el_ea))
 		goto out;
 
-	er.er_data = kmalloc(GFS_EA_DATA_LEN(el.el_ea), GFP_KERNEL);
+	er.er_data = kmalloc(GFS_EA_DATA_LEN(el.el_ea), GFP_NOFS);
 	error = -ENOMEM;
 	if (!er.er_data)
 		goto out;
@@ -212,7 +214,7 @@ gfs_acl_new_prep(struct gfs_inode *dip,
 	}
 
 	{
-		struct posix_acl *clone = posix_acl_clone(acl, GFP_KERNEL);
+		struct posix_acl *clone = posix_acl_clone(acl, GFP_NOFS);
 		error = -ENOMEM;
 		if (!clone)
 			goto out;
@@ -248,7 +250,7 @@ gfs_acl_new_prep(struct gfs_inode *dip,
 			b *= 2;
 		b++;
 
-		d = kmalloc(s, GFP_KERNEL);
+		d = kmalloc(s, GFP_NOFS);
 		error = -ENOMEM;
 		if (!d)
 			goto out;
@@ -353,7 +355,7 @@ gfs_acl_chmod(struct gfs_inode *ip, struct iattr *attr)
 	if (!GFS_EA_DATA_LEN(el.el_ea))
 		goto simple;
 
-	er.er_data = kmalloc(GFS_EA_DATA_LEN(el.el_ea), GFP_KERNEL);
+	er.er_data = kmalloc(GFS_EA_DATA_LEN(el.el_ea), GFP_NOFS);
 	error = -ENOMEM;
 	if (!er.er_data)
 		goto out;
