@@ -18,6 +18,10 @@ do { \
 } while (0)
 
 
+#define OUTPUT_BLOCKS 0
+#define OUTPUT_K      1
+#define OUTPUT_HUMAN  2
+
 extern char *prog_name;
 extern char *action;
 extern int override;
@@ -25,7 +29,7 @@ extern int expert;
 extern int debug;
 extern int continuous;
 extern int interval;
-
+extern int output_type;
 
 /* From counters.c */
 
@@ -82,9 +86,9 @@ void check_for_gfs(int fd, char *path);
 char *get_list(void);
 char **str2lines(char *str);
 char *mp2cookie(char *mp, int ioctl_ok);
-char *name2value(char *str, char *name);
-uint32_t name2u32(char *str, char *name);
-uint64_t name2u64(char *str, char *name);
+char *name2value(char *str, const char *name);
+uint32_t name2u32(char *str, const char *name);
+uint64_t name2u64(char *str, const char *name);
 
 
 #endif /* __GFS_TOOL_DOT_H__ */
