@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made availale to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,7 +11,6 @@
 #include <curses.h>
 
 #include "hexedit.h"
-#include "linux_endian.h"
 
 #define WANT_GFS_CONVERSION_FUNCTIONS
 #include <linux/gfs2_ondisk.h>
@@ -32,21 +18,95 @@
 #include "gfs2hex.h"
 /* from libgfs2: */
 #include "libgfs2.h"
-#include "ondisk.h"
+
+#define pv(struct, member, fmt, fmt2) do {				\
+		print_it("  "#member, fmt, fmt2, struct->member);	\
+	} while (FALSE);
+#define pv2(struct, member, fmt, fmt2) do {				\
+		print_it("  ", fmt, fmt2, struct->member);		\
+	} while (FALSE);
 
 
-extern struct gfs2_sb sb;
-extern char *buf;
-extern struct gfs2_dinode di;
-extern uint64_t bufsize;
-extern int line, termlines;
-extern char edit_fmt[80];
-extern char estring[1024];
-extern int edit_mode INIT(0);
-extern int edit_row[DMODES], edit_col[DMODES];
-extern int edit_size[DMODES], last_entry_onscreen[DMODES];
-extern char edit_fmt[80];
-extern enum dsp_mode dmode INIT(HEX_MODE); /* display mode */
+struct gfs2_sb sb;
+char *buf;
+struct gfs2_dinode di;
+int line, termlines;
+char edit_fmt[80];
+char estring[1024];
+char efield[64];
+int edit_mode = 0;
+int edit_row[DMODES], edit_col[DMODES];
+int edit_size[DMODES], last_entry_onscreen[DMODES];
+char edit_fmt[80];
+enum dsp_mode dmode = HEX_MODE; /* display mode */
+uint64_t block = 0;
+int blockhist = 0;
+struct iinfo *indirect;
+int indirect_blocks;
+int gfs1  = 0;
+uint64_t block_in_mem = -1;
+struct gfs2_sbd sbd;
+uint64_t starting_blk;
+struct blkstack_info blockstack[BLOCK_STACK_SIZE];
+int identify = FALSE;
+char device[NAME_MAX];
+uint64_t max_block = 0;
+int start_row[DMODES], end_row[DMODES], lines_per_row[DMODES];
+struct gfs_sb *sbd1;
+int gfs2_struct_type;
+unsigned int offset;
+int termcols = 80;
+struct indirect_info masterdir;
+struct gfs2_inum gfs1_quota_di;
+int print_entry_ndx;
+struct gfs2_inum gfs1_license_di;
+int screen_chunk_size = 512;
+uint64_t temp_blk;
+int color_scheme = 0;
+int struct_len;
+uint64_t dev_offset = 0;
+int editing = 0;
+int insert = 0;
+const char *termtype;
+WINDOW *wind;
+int dsplines = 0;
+
+const char *block_type_str[15] = {
+	"Clump",
+	"Superblock",
+	"Resource Group Header",
+	"Resource Group Bitmap",
+	"Dinode",
+	"Indirect Block",
+	"Leaf",
+	"Journaled Data",
+	"Log Header",
+	"Log descriptor",
+	"Ext. attrib",
+	"Eattr Data",
+	"Log Buffer",
+	"Metatype 13",
+	"Quota Change",
+};
+
+struct gfs1_rgrp {
+        struct gfs2_meta_header rg_header;
+
+        uint32_t rg_flags;      /* ?? */
+
+        uint32_t rg_free;       /* Number (qty) of free data blocks */
+
+        /* Dinodes are USEDMETA, but are handled separately from other METAs */
+        uint32_t rg_useddi;     /* Number (qty) of dinodes (used or free) */
+        uint32_t rg_freedi;     /* Number (qty) of unused (free) dinodes */
+        struct gfs2_inum rg_freedi_list; /* 1st block in chain of free dinodes */
+
+        /* These META statistics do not include dinodes (used or free) */
+        uint32_t rg_usedmeta;   /* Number (qty) of used metadata blocks */
+        uint32_t rg_freemeta;   /* Number (qty) of unused metadata blocks */
+
+        char rg_reserved[64];
+};
 
 void eol(int col) /* end of line */
 {
@@ -61,7 +121,7 @@ void eol(int col) /* end of line */
 	}
 }
 
-void print_gfs2(const char *fmt, ...)
+void __attribute__((format (printf, 1, 2))) print_gfs2(const char *fmt, ...)
 {
 	va_list args;
 	char string[NAME_MAX];
@@ -76,7 +136,7 @@ void print_gfs2(const char *fmt, ...)
 	va_end(args);
 }
 
-void check_highlight(int highlight)
+static void check_highlight(int highlight)
 {
 	if (!termlines || line >= termlines) /* If printing or out of bounds */
 		return;
@@ -167,6 +227,7 @@ void print_it(const char *label, const char *fmt, const char *fmt2, ...)
 		if (termlines) {
 			refresh();
 			if (line == (edit_row[dmode] * lines_per_row[dmode]) + 4) {
+				strcpy(efield, label + 2); /* it's indented */
 				strcpy(estring, tmp_string);
 				strcpy(edit_fmt, fmt);
 				edit_size[dmode] = strlen(estring);
@@ -179,7 +240,7 @@ void print_it(const char *label, const char *fmt, const char *fmt2, ...)
 	}
 }
 
-int indirect_dirent(struct indirect_info *indir, char *ptr, int d)
+static int indirect_dirent(struct indirect_info *indir, char *ptr, int d)
 {
 	struct gfs2_dirent de;
 
@@ -215,55 +276,58 @@ int indirect_dirent(struct indirect_info *indir, char *ptr, int d)
 **
 *******************************************************************************
 ******************************************************************************/
-void do_dinode_extended(struct gfs2_dinode *di, char *buf)
+void do_dinode_extended(struct gfs2_dinode *dine, char *dinebuf)
 {
-	unsigned int x, y;
+	unsigned int x, y, ptroff = 0;
 	uint64_t p, last;
-	int isdir = !!(S_ISDIR(di->di_mode)) || 
-		(gfs1 && di->__pad1 == GFS_FILE_DIR);
+	int isdir = !!(S_ISDIR(dine->di_mode)) || 
+		(gfs1 && dine->__pad1 == GFS_FILE_DIR);
 
 	indirect_blocks = 0;
 	memset(indirect, 0, sizeof(indirect));
-	if (di->di_height > 0) {
+	if (dine->di_height > 0) {
 		/* Indirect pointers */
-		for (x = sizeof(struct gfs2_dinode); x < bufsize;
+		for (x = sizeof(struct gfs2_dinode); x < sbd.bsize;
 			 x += sizeof(uint64_t)) {
-			p = be64_to_cpu(*(uint64_t *)(buf + x));
+			p = be64_to_cpu(*(uint64_t *)(dinebuf + x));
 			if (p) {
 				indirect->ii[indirect_blocks].block = p;
+				indirect->ii[indirect_blocks].mp.mp_list[0] =
+					ptroff;
 				indirect->ii[indirect_blocks].is_dir = FALSE;
 				indirect_blocks++;
 			}
+			ptroff++;
 		}
 	}
-	else if (isdir && !(di->di_flags & GFS2_DIF_EXHASH)) {
+	else if (isdir && !(dine->di_flags & GFS2_DIF_EXHASH)) {
 		int skip = 0;
 
 		/* Directory Entries: */
 		indirect->ii[0].dirents = 0;
 		indirect->ii[0].block = block;
 		indirect->ii[0].is_dir = TRUE;
-		for (x = sizeof(struct gfs2_dinode); x < bufsize; x += skip) {
+		for (x = sizeof(struct gfs2_dinode); x < sbd.bsize; x += skip) {
 			skip = indirect_dirent(indirect->ii,
-					       buf + x,
+					       dinebuf + x,
 					       indirect->ii[0].dirents);
 			if (skip <= 0)
 				break;
 		}
 	}
 	else if (isdir &&
-			 (di->di_flags & GFS2_DIF_EXHASH) &&
-			 di->di_height == 0) {
+			 (dine->di_flags & GFS2_DIF_EXHASH) &&
+			 dine->di_height == 0) {
 		/* Leaf Pointers: */
 		
-		last = be64_to_cpu(*(uint64_t *)(buf + sizeof(struct gfs2_dinode)));
+		last = be64_to_cpu(*(uint64_t *)(dinebuf + sizeof(struct gfs2_dinode)));
     
 		for (x = sizeof(struct gfs2_dinode), y = 0;
-			 y < (1 << di->di_depth);
+			 y < (1 << dine->di_depth);
 			 x += sizeof(uint64_t), y++) {
-			p = be64_to_cpu(*(uint64_t *)(buf + x));
+			p = be64_to_cpu(*(uint64_t *)(dinebuf + x));
 
-			if (p != last || ((y + 1) * sizeof(uint64_t) == di->di_size)) {
+			if (p != last || ((y + 1) * sizeof(uint64_t) == dine->di_size)) {
 				struct gfs2_buffer_head *tmp_bh;
 				int skip = 0, direntcount = 0;
 				struct gfs2_leaf leaf;
@@ -271,11 +335,11 @@ void do_dinode_extended(struct gfs2_dinode *di, char *buf)
 
 				if (last >= max_block)
 					break;
-				tmp_bh = bread(&sbd, last);
+				tmp_bh = bread(&sbd.buf_list, last);
 				gfs2_leaf_in(&leaf, tmp_bh->b_data);
 				indirect->ii[indirect_blocks].dirents = 0;
 				for (direntcount = 0, bufoffset = sizeof(struct gfs2_leaf);
-					 bufoffset < bufsize;
+					 bufoffset < sbd.bsize;
 					 direntcount++, bufoffset += skip) {
 					skip = indirect_dirent(&indirect->ii[indirect_blocks],
 										   tmp_bh->b_data + bufoffset,
@@ -307,21 +371,28 @@ void do_dinode_extended(struct gfs2_dinode *di, char *buf)
 **
 *******************************************************************************
 ******************************************************************************/
-int do_indirect_extended(char *buf, struct iinfo *iinf)
+int do_indirect_extended(char *diebuf, struct iinfo *iinf, int hgt)
 {
 	unsigned int x, y;
 	uint64_t p;
 	int i_blocks;
 
 	i_blocks = 0;
-	memset(iinf, 0, sizeof(struct iinfo));
+	for (x = 0; x < 512; x++) {
+		iinf->ii[x].is_dir = 0;
+		iinf->ii[x].height = 0;
+		iinf->ii[x].block = 0;
+		iinf->ii[x].dirents = 0;
+		memset(&iinf->ii[x].dirent, 0, sizeof(struct gfs2_dirents));
+	}
 	for (x = (gfs1 ? sizeof(struct gfs_indirect):
 			  sizeof(struct gfs2_meta_header)), y = 0;
-		 x < bufsize;
+		 x < sbd.bsize;
 		 x += sizeof(uint64_t), y++) {
-		p = be64_to_cpu(*(uint64_t *)(buf + x));
+		p = be64_to_cpu(*(uint64_t *)(diebuf + x));
 		if (p) {
 			iinf->ii[i_blocks].block = p;
+			iinf->ii[i_blocks].mp.mp_list[hgt] = i_blocks;
 			iinf->ii[i_blocks].is_dir = FALSE;
 			i_blocks++;
 		}
@@ -344,24 +415,24 @@ int do_indirect_extended(char *buf, struct iinfo *iinf)
 **
 *******************************************************************************
 ******************************************************************************/
-void do_leaf_extended(char *buf, struct iinfo *indir)
+void do_leaf_extended(char *dlebuf, struct iinfo *indir)
 {
 	int x, i;
 	struct gfs2_dirent de;
 
 	x = 0;
-	memset(indir, 0, sizeof(indir));
+	memset(indir, 0, sizeof(*indir));
 	/* Directory Entries: */
-	for (i = sizeof(struct gfs2_leaf); i < bufsize;
+	for (i = sizeof(struct gfs2_leaf); i < sbd.bsize;
 	     i += de.de_rec_len) {
-		gfs2_dirent_in(&de, buf + i);
+		gfs2_dirent_in(&de, dlebuf + i);
 		if (de.de_inum.no_addr) {
 			indir->ii[0].block = de.de_inum.no_addr;
 			indir->ii[0].dirent[x].block = de.de_inum.no_addr;
 			memcpy(&indir->ii[0].dirent[x].dirent,
 			       &de, sizeof(struct gfs2_dirent));
 			memcpy(&indir->ii[0].dirent[x].filename,
-			       buf + i + sizeof(struct gfs2_dirent),
+			       dlebuf + i + sizeof(struct gfs2_dirent),
 			       de.de_name_len);
 			indir->ii[0].dirent[x].filename[de.de_name_len] = '\0';
 			indir->ii[0].is_dir = TRUE;
@@ -390,7 +461,7 @@ void do_leaf_extended(char *buf, struct iinfo *indir)
 *******************************************************************************
 ******************************************************************************/
 
-void do_eattr_extended(char *buf)
+static void do_eattr_extended(char *deebuf)
 {
 	struct gfs2_ea_header ea;
 	unsigned int x;
@@ -399,15 +470,15 @@ void do_eattr_extended(char *buf)
 	print_gfs2("Eattr Entries:");
 	eol(0);
 
-	for (x = sizeof(struct gfs2_meta_header); x < bufsize; x += ea.ea_rec_len)
+	for (x = sizeof(struct gfs2_meta_header); x < sbd.bsize; x += ea.ea_rec_len)
 	{
 		eol(0);
-		gfs2_ea_header_in(&ea, buf + x);
-		gfs2_ea_header_print(&ea, buf + x + sizeof(struct gfs2_ea_header));
+		gfs2_ea_header_in(&ea, deebuf + x);
+		gfs2_ea_header_print(&ea, deebuf + x + sizeof(struct gfs2_ea_header));
 	}
 }
 
-void gfs2_inum_print2(const char *title,struct gfs2_inum *no)
+static void gfs2_inum_print2(const char *title,struct gfs2_inum *no)
 {
 	if (termlines) {
 		check_highlight(TRUE);
@@ -427,29 +498,69 @@ void gfs2_inum_print2(const char *title,struct gfs2_inum *no)
  * gfs2_sb_print2 - Print out a superblock
  * @sb: the cpu-order buffer
  */
-void gfs2_sb_print2(struct gfs2_sb *sb)
+static void gfs2_sb_print2(struct gfs2_sb *sbp2)
 {
-	gfs2_meta_header_print(&sb->sb_header);
+	gfs2_meta_header_print(&sbp2->sb_header);
 
-	pv(sb, sb_fs_format, "%u", "0x%x");
-	pv(sb, sb_multihost_format, "%u", "0x%x");
+	pv(sbp2, sb_fs_format, "%u", "0x%x");
+	pv(sbp2, sb_multihost_format, "%u", "0x%x");
 
-	pv(sb, sb_bsize, "%u", "0x%x");
-	pv(sb, sb_bsize_shift, "%u", "0x%x");
+	if (gfs1)
+		pv(sbd1, sb_flags, "%u", "0x%x");
+	pv(sbp2, sb_bsize, "%u", "0x%x");
+	pv(sbp2, sb_bsize_shift, "%u", "0x%x");
 	if (gfs1) {
+		pv(sbd1, sb_seg_size, "%u", "0x%x");
 		gfs2_inum_print2("jindex ino", &sbd1->sb_jindex_di);
 		gfs2_inum_print2("rindex ino", &sbd1->sb_rindex_di);
 	}
 	else
-		gfs2_inum_print2("master dir", &sb->sb_master_dir);
-	gfs2_inum_print2("root dir  ", &sb->sb_root_dir);
+		gfs2_inum_print2("master dir", &sbp2->sb_master_dir);
+	gfs2_inum_print2("root dir  ", &sbp2->sb_root_dir);
 
-	pv(sb, sb_lockproto, "%s", NULL);
-	pv(sb, sb_locktable, "%s", NULL);
+	pv(sbp2, sb_lockproto, "%s", NULL);
+	pv(sbp2, sb_locktable, "%s", NULL);
 	if (gfs1) {
 		gfs2_inum_print2("quota ino ", &gfs1_quota_di);
 		gfs2_inum_print2("license   ", &gfs1_license_di);
 	}
+#ifdef GFS2_HAS_UUID
+	print_it("  sb_uuid", "%s", NULL, str_uuid(sbp2->sb_uuid));
+#endif
+}
+
+/**
+ * gfs1_rgrp_in - read in a gfs1 rgrp
+ */
+static void gfs1_rgrp_in(struct gfs1_rgrp *rgrp, char *rbuf)
+{
+        struct gfs1_rgrp *str = (struct gfs1_rgrp *)rbuf;
+
+        gfs2_meta_header_in(&rgrp->rg_header, rbuf);
+        rgrp->rg_flags = be32_to_cpu(str->rg_flags);
+        rgrp->rg_free = be32_to_cpu(str->rg_free);
+        rgrp->rg_useddi = be32_to_cpu(str->rg_useddi);
+        rgrp->rg_freedi = be32_to_cpu(str->rg_freedi);
+        gfs2_inum_in(&rgrp->rg_freedi_list, (char *)&str->rg_freedi_list);
+        rgrp->rg_usedmeta = be32_to_cpu(str->rg_usedmeta);
+        rgrp->rg_freemeta = be32_to_cpu(str->rg_freemeta);
+        memcpy(rgrp->rg_reserved, str->rg_reserved, 64);
+}
+
+/**
+ * gfs_rgrp_print - Print out a resource group header
+ */
+static void gfs1_rgrp_print(struct gfs1_rgrp *rg)
+{
+        gfs2_meta_header_print(&rg->rg_header);
+        pv(rg, rg_flags, "%u", "0x%x");
+        pv(rg, rg_free, "%u", "0x%x");
+        pv(rg, rg_useddi, "%u", "0x%x");
+        pv(rg, rg_freedi, "%u", "0x%x");
+        gfs2_inum_print(&rg->rg_freedi_list);
+
+        pv(rg, rg_usedmeta, "%u", "0x%x");
+        pv(rg, rg_freemeta, "%u", "0x%x");
 }
 
 /******************************************************************************
@@ -487,57 +598,53 @@ int display_gfs2(void)
 	{
 	case GFS2_MAGIC:
 		gfs2_meta_header_in(&mh, buf);
+		if (mh.mh_type > GFS2_METATYPE_QC)
+			print_gfs2("Unknown metadata type");
+		else
+			print_gfs2("%s:", block_type_str[mh.mh_type]);
+		eol(0);
 		
 		switch (mh.mh_type)
 		{
 		case GFS2_METATYPE_SB:
-			print_gfs2("Superblock:");
-			eol(0);
 			gfs2_sb_in(&sbd.sd_sb, buf);
 			gfs2_sb_print2(&sbd.sd_sb);
 			break;
-			
+
 		case GFS2_METATYPE_RG:
-			print_gfs2("Resource Group Header:");
-			eol(0);
-			gfs2_rgrp_in(&rg, buf);
-			gfs2_rgrp_print(&rg);
+			if (gfs1) {
+				struct gfs1_rgrp rg1;
+
+				gfs1_rgrp_in(&rg1, buf);
+				gfs1_rgrp_print(&rg1);
+			} else {
+				gfs2_rgrp_in(&rg, buf);
+				gfs2_rgrp_print(&rg);
+			}
 			break;
-			
+
 		case GFS2_METATYPE_RB:
-			print_gfs2("Resource Group Bitmap:");
-			eol(0);
 			gfs2_meta_header_print(&mh);
 			break;
-			
+
 		case GFS2_METATYPE_DI:
-			print_gfs2("Dinode:");
-			eol(0);
 			gfs2_dinode_print(&di);
 			break;
-			
+
+		case GFS2_METATYPE_IN:
+			gfs2_meta_header_print(&mh);
+			break;
+
 		case GFS2_METATYPE_LF:
-			print_gfs2("Leaf:");
-			eol(0);
 			gfs2_leaf_in(&lf, buf);
 			gfs2_leaf_print(&lf);
 			break;
-			
-		case GFS2_METATYPE_IN:
-			print_gfs2("Indirect Block:");
-			eol(0);
-			gfs2_meta_header_print(&mh);
-			break;
-			
+
 		case GFS2_METATYPE_JD:
-			print_gfs2("Journaled File Block:");
-			eol(0);
 			gfs2_meta_header_print(&mh);
 			break;
-			
+
 		case GFS2_METATYPE_LH:
-			print_gfs2("Log Header:");
-			eol(0);
 			if (gfs1) {
 				gfs_log_header_in(&lh1, buf);
 				gfs_log_header_print(&lh1);
@@ -546,42 +653,30 @@ int display_gfs2(void)
 				gfs2_log_header_print(&lh);
 			}
 			break;
-			
+
 		case GFS2_METATYPE_LD:
-			print_gfs2("Log descriptor");
-			eol(0);
 			gfs2_log_descriptor_in(&ld, buf);
 			gfs2_log_descriptor_print(&ld);
 			break;
 
 		case GFS2_METATYPE_EA:
-			print_gfs2("Eattr Block:");
-			eol(0);
 			do_eattr_extended(buf);
 			break;
 			
 		case GFS2_METATYPE_ED:
-			print_gfs2("Eattr Data Block:");
-			eol(0);
 			gfs2_meta_header_print(&mh);
 			break;
 			
 		case GFS2_METATYPE_LB:
-			print_gfs2("Log Buffer");
-			eol(0);
 			gfs2_meta_header_print(&mh);
 			break;
 
 		case GFS2_METATYPE_QC:
-			print_gfs2("Quota Change");
-			eol(0);
 			gfs2_quota_change_in(&qc, buf);
 			gfs2_quota_change_print(&qc);
 			break;
 
 		default:
-			print_gfs2("Unknown metadata type");
-			eol(0);
 			break;
 		}
 		break;
