@@ -7,6 +7,9 @@
 #include <stdarg.h>
 #include <ctype.h>
 #include <signal.h>
+#include <libintl.h>
+#include <locale.h>
+#define _(String) gettext(String)
 
 #include "copyright.cf"
 #include "libgfs2.h"
@@ -17,13 +20,14 @@ struct gfs2_options opts = {0};
 struct gfs2_inode *lf_dip; /* Lost and found directory inode */
 osi_list_t dir_hash[FSCK_HASH_SIZE];
 osi_list_t inode_hash[FSCK_HASH_SIZE];
-struct gfs2_block_list *bl;
+struct gfs2_block_list *bl = NULL;
 uint64_t last_fs_block, last_reported_block = -1;
 int skip_this_pass = FALSE, fsck_abort = FALSE;
+int errors_found = 0, errors_corrected = 0;
 const char *pass = "";
 uint64_t last_data_block;
 uint64_t first_data_block;
-char *prog_name = "gfs2_fsck"; /* needed by libgfs2 */
+int preen = 0, force_check = 0;
 
 /* This function is for libgfs2's sake.                                      */
 void print_it(const char *label, const char *fmt, const char *fmt2, ...)
@@ -36,31 +40,42 @@ void print_it(const char *label, const char *fmt, const char *fmt2, ...)
 	va_end(args);
 }
 
-void usage(char *name)
+static void usage(char *name)
 {
-	printf("Usage: %s [-hnqvVy] <device> \n", basename(name));
+	printf("Usage: %s [-afhnpqvVy] <device> \n", basename(name));
 }
 
-void version(void)
+static void version(void)
 {
-	printf("GFS2 fsck %s (built %s %s)\n",
+	printf( _("GFS2 fsck %s (built %s %s)\n"),
 	       RELEASE_VERSION, __DATE__, __TIME__);
-	printf("%s\n", REDHAT_COPYRIGHT);
+	printf( _(REDHAT_COPYRIGHT "\n"));
 }
 
-int read_cmdline(int argc, char **argv, struct gfs2_options *opts)
+static int read_cmdline(int argc, char **argv, struct gfs2_options *gopts)
 {
 	int c;
 
-	while((c = getopt(argc, argv, "hnqvyV")) != -1) {
+	while((c = getopt(argc, argv, "afhnpqvyV")) != -1) {
 		switch(c) {
 
+		case 'a':
+			preen = 1;
+			gopts->yes = 1;
+			break;
+		case 'f':
+			force_check = 1;
+			break;
 		case 'h':
 			usage(argv[0]);
-			exit(0);
+			exit(FSCK_OK);
 			break;
 		case 'n':
-			opts->no = 1;
+			gopts->no = 1;
+			break;
+		case 'p':
+			preen = 1;
+			gopts->yes = 1;
 			break;
 		case 'q':
 			decrease_verbosity();
@@ -70,52 +85,50 @@ int read_cmdline(int argc, char **argv, struct gfs2_options *opts)
 			break;
 		case 'V':
 			version();
-			exit(0);
+			exit(FSCK_OK);
 			break;
 		case 'y':
-			opts->yes = 1;
+			gopts->yes = 1;
 			break;
 		case ':':
 		case '?':
-			fprintf(stderr, "Please use '-h' for usage.\n");
-			exit(1);
-			break;
+			fprintf(stderr, _("Please use '-h' for usage.\n"));
+			return FSCK_USAGE;
 		default:
-			fprintf(stderr, "Bad programmer! You forgot to catch"
-				" the %c flag\n", c);
-			exit(1);
-			break;
+			fprintf(stderr, _("Bad programmer! You forgot to catch"
+				" the %c flag\n"), c);
+			return FSCK_USAGE;
 
 		}
 	}
 	if(argc > optind) {
-		opts->device = (argv[optind]);
-		if(!opts->device) {
-			fprintf(stderr, "Please use '-h' for usage.\n");
-			exit(1);
+		gopts->device = (argv[optind]);
+		if(!gopts->device) {
+			fprintf(stderr, _("Please use '-h' for usage.\n"));
+			return FSCK_USAGE;
 		}
 	} else {
-		fprintf(stderr, "No device specified.  Use '-h' for usage.\n");
-		exit(1);
+		fprintf(stderr, _("No device specified.  Use '-h' for usage.\n"));
+		return FSCK_USAGE;
 	}
 	return 0;
 }
 
-void interrupt(int sig)
+static void interrupt(int sig)
 {
 	char response;
 	char progress[PATH_MAX];
 
 	if (!last_reported_block || last_reported_block == last_fs_block)
-		sprintf(progress, "progress unknown.\n");
+		sprintf(progress, _("progress unknown.\n"));
 	else
-		sprintf(progress, "processing block %" PRIu64 " out of %"
-			PRIu64 "\n", last_reported_block, last_fs_block);
+		sprintf(progress, _("processing block %" PRIu64 " out of %"
+			PRIu64 "\n"), last_reported_block, last_fs_block);
 	
 	response = generic_interrupt("gfs2_fsck", pass, progress,
-				     "Do you want to abort gfs2_fsck, skip " \
+				     _("Do you want to abort gfs2_fsck, skip " \
 				     "the rest of this pass or continue " \
-				     "(a/s/c)?", "asc");
+				     "(a/s/c)?"), "asc");
 	if(tolower(response) == 's') {
 		skip_this_pass = TRUE;
 		return;
@@ -128,27 +141,27 @@ void interrupt(int sig)
 
 /* Check system inode and verify it's marked "in use" in the bitmap:       */
 /* Should work for all system inodes: root, master, jindex, per_node, etc. */
-int check_system_inode(struct gfs2_inode *sysinode, const char *filename,
-		       void builder(struct gfs2_sbd *sbp),
+static int check_system_inode(struct gfs2_inode *sysinode, const char *filename,
+		       int builder(struct gfs2_sbd *sbp),
 		       enum gfs2_mark_block mark)
 {
 	uint64_t iblock = 0;
 	struct dir_status ds = {0};
 
-	log_info("Checking system inode '%s'\n", filename);
+	log_info( _("Checking system inode '%s'\n"), filename);
 	if (sysinode) {
 		/* Read in the system inode, look at its dentries, and start
 		 * reading through them */
 		iblock = sysinode->i_di.di_num.no_addr;
-		log_info("System inode for '%s' is located at block %"
-			 PRIu64 " (0x%" PRIx64 ")\n", filename,
+		log_info( _("System inode for '%s' is located at block %"
+			 PRIu64 " (0x%" PRIx64 ")\n"), filename,
 			 iblock, iblock);
 		
 		/* FIXME: check this block's validity */
 
 		if(gfs2_block_check(sysinode->i_sbd, bl, iblock, &ds.q)) {
-			log_crit("Can't get %s inode block %" PRIu64 " (0x%"
-				 PRIx64 ") from block list\n", filename,
+			log_crit( _("Can't get %s inode block %" PRIu64 " (0x%"
+				 PRIx64 ") from block list\n"), filename,
 				 iblock, iblock);
 			return -1;
 		}
@@ -157,7 +170,7 @@ int check_system_inode(struct gfs2_inode *sysinode, const char *filename,
 		/* bitmap.  In that case, don't rebuild the inode.  */
 		/* Just reuse the inode and fix the bitmap.         */
 		if (ds.q.block_type == gfs2_block_free) {
-			log_info("The inode exists but the block is not marked 'in use'; fixing it.\n");
+			log_info( _("The inode exists but the block is not marked 'in use'; fixing it.\n"));
 			gfs2_block_set(sysinode->i_sbd, bl,
 				       sysinode->i_di.di_num.no_addr,
 				       mark);
@@ -168,15 +181,17 @@ int check_system_inode(struct gfs2_inode *sysinode, const char *filename,
 		}
 	}
 	else
-		log_info("System inode for '%s' is missing.\n", filename);
+		log_info( _("System inode for '%s' is missing.\n"), filename);
 	/* If there are errors with the inode here, we need to
 	 * create a new inode and get it all setup - of course,
 	 * everything will be in lost+found then, but we *need* our
 	 * system inodes before we can do any of that. */
 	if(!sysinode || ds.q.block_type != mark) {
-		log_err("Invalid or missing %s system inode.\n", filename);
-		if (query(&opts, "Create new %s system inode? (y/n) ",
+		log_err( _("Invalid or missing %s system inode.\n"), filename);
+		errors_found++;
+		if (query(&opts, _("Create new %s system inode? (y/n) "),
 			  filename)) {
+			errors_corrected++;
 			builder(sysinode->i_sbd);
 			gfs2_block_set(sysinode->i_sbd, bl,
 				       sysinode->i_di.di_num.no_addr,
@@ -187,7 +202,7 @@ int check_system_inode(struct gfs2_inode *sysinode, const char *filename,
 						sysinode->i_di.di_num.no_addr);
 		}
 		else {
-			log_err("Cannot continue without valid %s inode\n",
+			log_err( _("Cannot continue without valid %s inode\n"),
 				filename);
 			return -1;
 		}
@@ -196,7 +211,7 @@ int check_system_inode(struct gfs2_inode *sysinode, const char *filename,
 	return 0;
 }
 
-int check_system_inodes(struct gfs2_sbd *sdp)
+static int check_system_inodes(struct gfs2_sbd *sdp)
 {
 	/*******************************************************************
 	 *******  Check the system inode integrity             *************
@@ -244,34 +259,109 @@ int check_system_inodes(struct gfs2_sbd *sdp)
 	return 0;
 }
 
+static void check_statfs(struct gfs2_sbd *sdp)
+{
+	osi_list_t *tmp;
+	struct rgrp_list *rgd;
+	struct gfs2_rindex *ri;
+	struct gfs2_statfs_change sc;
+	char buf[sizeof(struct gfs2_statfs_change)];
+	int count;
+
+	/* Read the current statfs values */
+	count = gfs2_readi(sdp->md.statfs, buf, 0,
+			   sdp->md.statfs->i_di.di_size);
+	if (count == sizeof(struct gfs2_statfs_change))
+		gfs2_statfs_change_in(&sc, buf);
+
+	/* Calculate the real values from the rgrp information */
+	sdp->blks_total = 0;
+	sdp->blks_alloced = 0;
+	sdp->dinodes_alloced = 0;
+
+	for (tmp = sdp->rglist.next; tmp != &sdp->rglist; tmp = tmp->next) {
+		rgd = osi_list_entry(tmp, struct rgrp_list, list);
+		ri = &rgd->ri;
+		sdp->blks_total += ri->ri_data;
+		sdp->blks_alloced += (ri->ri_data - rgd->rg.rg_free);
+		sdp->dinodes_alloced += rgd->rg.rg_dinodes;
+	}
+
+	/* See if they match */
+	if (sc.sc_total == sdp->blks_total &&
+	    sc.sc_free == (sdp->blks_total - sdp->blks_alloced) &&
+	    sc.sc_dinodes == sdp->dinodes_alloced) {
+		log_info( _("The statfs file is accurate.\n"));
+		return;
+	}
+	log_err( _("The statfs file is wrong:\n\n"));
+	log_err( _("Current statfs values:\n"));
+	log_err( _("blocks:  %lld (0x%llx)\n"),
+		sc.sc_total, sc.sc_total);
+	log_err( _("free:    %lld (0x%llx)\n"),
+		sc.sc_free, sc.sc_free);
+	log_err( _("dinodes: %lld (0x%llx)\n\n"),
+		sc.sc_dinodes, sc.sc_dinodes);
+
+	log_err( _("Calculated statfs values:\n"));
+	log_err( _("blocks:  %lld (0x%llx)\n"),
+		sdp->blks_total, sdp->blks_total);
+	log_err( _("free:    %lld (0x%llx)\n"),
+		sdp->blks_total - sdp->blks_alloced,
+		sdp->blks_total - sdp->blks_alloced);
+	log_err( _("dinodes: %lld (0x%llx)\n"),
+		sdp->dinodes_alloced, sdp->dinodes_alloced);
+
+	errors_found++;
+	if (!query(&opts, _("Okay to fix the master statfs file? (y/n)"))) {
+		log_err( _("The statfs file was not fixed.\n"));
+		return;
+	}
+
+	do_init_statfs(sdp);
+	log_err( _("The statfs file was fixed.\n"));
+	errors_corrected++;
+}
+
 int main(int argc, char **argv)
 {
 	struct gfs2_sbd sb;
 	struct gfs2_sbd *sbp = &sb;
 	int j;
 	enum update_flags update_sys_files;
+	int error = 0;
+	int all_clean = 0;
+
+	setlocale(LC_ALL, "");
+	textdomain("gfs2-utils");
 
 	memset(sbp, 0, sizeof(*sbp));
 
-	if(read_cmdline(argc, argv, &opts))
-		return 1;
+	if((error = read_cmdline(argc, argv, &opts)))
+		exit(error);
 	setbuf(stdout, NULL);
-	log_notice("Initializing fsck\n");
-	if (initialize(sbp))
-		return 1;
+	log_notice( _("Initializing fsck\n"));
+	if ((error = initialize(sbp, force_check, preen, &all_clean)))
+		exit(error);
+
+	if (!force_check && all_clean && preen) {
+		log_err( _("%s: clean.\n"), opts.device);
+		destroy(sbp);
+		exit(FSCK_OK);
+	}
 
 	signal(SIGINT, interrupt);
-	log_notice("Starting pass1\n");
+	log_notice( _("Starting pass1\n"));
 	pass = "pass 1";
 	last_reported_block = 0;
-	if (pass1(sbp))
-		return 1;
+	if ((error = pass1(sbp)))
+		exit(error);
 	if (skip_this_pass || fsck_abort) {
 		skip_this_pass = FALSE;
-		log_notice("Pass1 interrupted   \n");
+		log_notice( _("Pass1 interrupted   \n"));
 	}
 	else
-		log_notice("Pass1 complete      \n");
+		log_notice( _("Pass1 complete      \n"));
 
 	/* Make sure the system inodes are okay & represented in the bitmap. */
 	check_system_inodes(sbp);
@@ -279,82 +369,88 @@ int main(int argc, char **argv)
 	if (!fsck_abort) {
 		last_reported_block = 0;
 		pass = "pass 1b";
-		log_notice("Starting pass1b\n");
-		if(pass1b(sbp))
-			return 1;
+		log_notice( _("Starting pass1b\n"));
+		if((error = pass1b(sbp)))
+			exit(error);
 		if (skip_this_pass || fsck_abort) {
 			skip_this_pass = FALSE;
-			log_notice("Pass1b interrupted   \n");
+			log_notice( _("Pass1b interrupted   \n"));
 		}
 		else
-			log_notice("Pass1b complete\n");
+			log_notice( _("Pass1b complete\n"));
 	}
 	if (!fsck_abort) {
 		last_reported_block = 0;
 		pass = "pass 1c";
-		log_notice("Starting pass1c\n");
-		if(pass1c(sbp))
-			return 1;
+		log_notice( _("Starting pass1c\n"));
+		if((error = pass1c(sbp)))
+			exit(error);
 		if (skip_this_pass || fsck_abort) {
 			skip_this_pass = FALSE;
-			log_notice("Pass1c interrupted   \n");
+			log_notice( _("Pass1c interrupted   \n"));
 		}
 		else
-			log_notice("Pass1c complete\n");
+			log_notice( _("Pass1c complete\n"));
 	}
 	if (!fsck_abort) {
 		last_reported_block = 0;
 		pass = "pass 2";
-		log_notice("Starting pass2\n");
-		if (pass2(sbp))
-			return 1;
+		log_notice( _("Starting pass2\n"));
+		if ((error = pass2(sbp)))
+			exit(error);
 		if (skip_this_pass || fsck_abort) {
 			skip_this_pass = FALSE;
-			log_notice("Pass2 interrupted   \n");
+			log_notice( _("Pass2 interrupted   \n"));
 		}
 		else
-			log_notice("Pass2 complete      \n");
+			log_notice( _("Pass2 complete      \n"));
 	}
 	if (!fsck_abort) {
 		last_reported_block = 0;
 		pass = "pass 3";
-		log_notice("Starting pass3\n");
-		if (pass3(sbp))
-			return 1;
+		log_notice( _("Starting pass3\n"));
+		if ((error = pass3(sbp)))
+			exit(error);
 		if (skip_this_pass || fsck_abort) {
 			skip_this_pass = FALSE;
-			log_notice("Pass3 interrupted   \n");
+			log_notice( _("Pass3 interrupted   \n"));
 		}
 		else
-			log_notice("Pass3 complete      \n");
+			log_notice( _("Pass3 complete      \n"));
 	}
 	if (!fsck_abort) {
 		last_reported_block = 0;
 		pass = "pass 4";
-		log_notice("Starting pass4\n");
-		if (pass4(sbp))
-			return 1;
+		log_notice( _("Starting pass4\n"));
+		if ((error = pass4(sbp)))
+			exit(error);
 		if (skip_this_pass || fsck_abort) {
 			skip_this_pass = FALSE;
-			log_notice("Pass4 interrupted   \n");
+			log_notice( _("Pass4 interrupted   \n"));
 		}
 		else
-			log_notice("Pass4 complete      \n");
+			log_notice( _("Pass4 complete      \n"));
 	}
 	if (!fsck_abort) {
 		last_reported_block = 0;
 		pass = "pass 5";
-		log_notice("Starting pass5\n");
-		if (pass5(sbp))
-			return 1;
+		log_notice( _("Starting pass5\n"));
+		if ((error = pass5(sbp)))
+			exit(error);
 		if (skip_this_pass || fsck_abort) {
 			skip_this_pass = FALSE;
-			log_notice("Pass5 interrupted   \n");
+			log_notice( _("Pass5 interrupted   \n"));
+			error = FSCK_CANCELED;
 		}
 		else
-			log_notice("Pass5 complete      \n");
+			log_notice( _("Pass5 complete      \n"));
+	} else {
+		error = FSCK_CANCELED;
 	}
 	update_sys_files = (opts.no ? not_updated : updated);
+
+	check_statfs(sbp);
+
 	/* Free up our system inodes */
 	inode_put(sbp->md.inum, update_sys_files);
 	inode_put(sbp->md.statfs, update_sys_files);
@@ -369,12 +465,21 @@ int main(int argc, char **argv)
 	if (lf_dip)
 		inode_put(lf_dip, update_sys_files);
 
-	if (!opts.no)
-		log_notice("Writing changes to disk\n");
+	if (!opts.no && errors_corrected)
+		log_notice( _("Writing changes to disk\n"));
+
 	bsync(&sbp->buf_list);
 	bsync(&sbp->nvbuf_list);
 	destroy(sbp);
-	log_notice("gfs2_fsck complete    \n");
+	log_notice( _("gfs2_fsck complete    \n"));
 
-	return 0;
+	if (!error) {
+		if (!errors_found)
+			error = FSCK_OK;
+		else if (errors_found == errors_corrected)
+			error = FSCK_NONDESTRUCT;
+		else
+			error = FSCK_UNCORRECTED;
+	}
+	exit(error);
 }
