@@ -2,7 +2,6 @@
 #define __GFS2_QUOTA_DOT_H__
 
 #include "libgfs2.h"
-#include "linux_endian.h"
 #include <linux/gfs2_ondisk.h>
 
 #define type_zalloc(ptr, type, count) \
@@ -59,21 +58,17 @@ struct commandline {
 };
 typedef struct commandline commandline_t;
 
-extern char *prog_name;
-
 /*  main.c  */
 
 void do_get_super(int fd, struct gfs2_sb *sb);
 void do_sync(struct gfs2_sbd *sdp, commandline_t *comline);
-void cleanup();
+void cleanup(void);
 void read_superblock(struct gfs2_sb *sb, struct gfs2_sbd *sdp);
 void get_last_quota_id(int fd, uint32_t *max_id);
 int is_valid_quota_list(int fd);
-inline void read_quota_internal(int fd, unsigned int id, int id_type, 
+void read_quota_internal(int fd, unsigned int id, int id_type,
 				struct gfs2_quota *q);
-inline void write_quota_internal(int fd, unsigned int id, int id_type, 
-				 struct gfs2_quota *q);
-void print_quota_list_warning();
+void print_quota_list_warning(void);
 
 /*  check.c  */
 
