@@ -30,15 +30,17 @@
 
 #define OP_LIST				1
 #define OP_DUMP				2
-#define OP_LOG				3
+#define OP_COMPAT			3
+
+#define DEFAULT_GROUPD_COMPAT		0
 
 static char *prog_name;
 static int operation;
 static int opt_ind;
 static int verbose;
-static int all_daemons;
 static int ls_all_nodes;
-static int print_header_done;
+static int opt_groupd_compat;
+static int cfg_groupd_compat = DEFAULT_GROUPD_COMPAT;
 
 
 static int do_write(int fd, void *buf, size_t count)
@@ -46,7 +48,7 @@ static int do_write(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
  retry:
-	rv = write(fd, buf + off, count);
+	rv = write(fd, (char *)buf + off, count);
 	if (rv == -1 && errno == EINTR)
 		goto retry;
 	if (rv < 0)
@@ -65,7 +67,7 @@ static int do_read(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
 	while (off < count) {
-		rv = read(fd, buf + off, count - off);
+		rv = read(fd, (char *)buf + off, count - off);
 		if (rv == 0)
 			return -1;
 		if (rv == -1 && errno == EINTR)
@@ -81,32 +83,37 @@ static void print_usage(void)
 {
 	printf("Usage:\n");
 	printf("\n");
-	printf("%s [options] [ls|dump]\n", prog_name);
+	printf("%s [options] [compat|ls|dump]\n", prog_name);
 	printf("\n");
 	printf("Options:\n");
-	printf("  -v               Verbose output, extra event information\n");
-	printf("  -a               fence_tool ls; dlm_tool ls; gfs_control ls\n");
-	printf("  -n               Show all node information with -a\n");
 	printf("  -h               Print this help, then exit\n");
 	printf("  -V               Print program version information, then exit\n");
 	printf("\n");
-	printf("Display group information from groupd\n");
-	printf("ls                 Show information for all groups\n");
-	printf("ls <level> <name>  Show information one group.  If\n");
-	printf("                   we are not a member of the group,\n");
-	printf("                   return 1.\n");
+
+	printf("compat             Show compatibility mode that groupd is running\n");
 	printf("\n");
-	printf("Display debugging information\n");
+
+	printf("ls                 Show group state for fence, dlm, gfs\n");
+	printf("   -g <num>        select daemons to query\n");
+	printf("                   0: query fenced, dlm_controld, gfs_controld\n");
+	printf("                   1: query groupd (for old compat mode)\n");
+	printf("                   2: use 0 if daemons running in new mode,\n");
+	printf("                   or 1 if groupd running in old mode.\n");
+	printf("                   Default %d\n", DEFAULT_GROUPD_COMPAT);
+	printf("   -n              Show all node information (with -g0)\n");
+	printf("   -v              Show extra event information (with -g1)\n");
+	printf("\n");
+
 	printf("dump               Show debug log from groupd\n");
-	printf("dump fence         Show debug log from fenced\n");
-	printf("dump gfs           Show debug log from gfs_controld\n");
-	printf("dump plocks <name> Show posix locks for gfs with given name\n");
-	printf("\n");
-	printf("log <comments>     Add information to the groupd log.\n");
+	printf("dump fence         Show debug log from fenced (fence_tool dump)\n");
+	printf("dump dlm           Show debug log from dlm_controld (dlm_tool dump)\n");
+	printf("dump gfs           Show debug log from gfs_controld (gfs_control dump)\n");
+	printf("dump plocks <name> Show posix locks from dlm_controld for lockspace <name>\n");
+	printf("                   (dlm_tool plocks <name>)\n");
 	printf("\n");
 }
 
-#define OPTION_STRING "ahVvn"
+#define OPTION_STRING "g:hVvn"
 
 static void decode_arguments(int argc, char **argv)
 {
@@ -117,8 +124,10 @@ static void decode_arguments(int argc, char **argv)
 		optchar = getopt(argc, argv, OPTION_STRING);
 
 		switch (optchar) {
-		case 'a':
-			all_daemons = 1;
+		
+		case 'g':
+			opt_groupd_compat = 1;
+			cfg_groupd_compat = atoi(optarg);
 			break;
 
 		case 'n':
@@ -168,8 +177,8 @@ static void decode_arguments(int argc, char **argv)
 			operation = OP_LIST;
 			opt_ind = optind + 1;
 			break;
-		} else if (strcmp(argv[optind], "log") == 0) {
-			operation = OP_LOG;
+		} else if (strcmp(argv[optind], "compat") == 0) {
+			operation = OP_COMPAT;
 			opt_ind = optind + 1;
 			break;
 		}
@@ -198,7 +207,7 @@ static void decode_arguments(int argc, char **argv)
 #define EST_FAIL_START_WAIT   14
 #define EST_FAIL_ALL_STARTED  15
 
-char *ev_state_str(state)
+static const char *ev_state_str(int state)
 {
 	switch (state) {
 	case EST_JOIN_BEGIN:
@@ -236,10 +245,10 @@ char *ev_state_str(state)
 	}
 }
 
-char *state_str(group_data_t *data)
+static const char *state_str(group_data_t *data)
 {
 	static char buf[128];
-	
+
 	memset(buf, 0, sizeof(buf));
 
 	if (!data->event_state && !data->event_nodeid)
@@ -274,7 +283,8 @@ static int groupd_list(int argc, char **argv)
 {
 	group_data_t data[MAX_GROUPS];
 	int i, j, rv, count = 0, level, ret = 0;
-	char *name, *state_header;
+	char *name;
+	const char *state_header;
 	int type_width = 16;
 	int level_width = 5;
 	int name_width = 32;
@@ -292,7 +302,8 @@ static int groupd_list(int argc, char **argv)
 		count = 1;
 
 		/* don't output if there's no group at all */
-		if (data[0].id == 0) {
+		if (data[0].id == 0 && !strlen(data[0].name) && 
+		    !strlen(data[0].client_name)) {
 			fprintf(stderr, "groupd has no information about "
 			        "the specified group\n");
 			return 1;
@@ -355,6 +366,9 @@ static int groupd_list(int argc, char **argv)
 	return ret;
 }
 
+#if 0
+static int print_header_done;
+
 static int fenced_node_compare(const void *va, const void *vb)
 {
 	const struct fenced_node *a = va;
@@ -543,8 +557,9 @@ static void gfs_controld_list(void)
 		printf("]\n");
 	}
 }
+#endif
 
-static int connect_daemon(char *path)
+static int connect_daemon(const char *path)
 {
 	struct sockaddr_un sun;
 	socklen_t addrlen;
@@ -586,36 +601,75 @@ static void groupd_dump_debug(int argc, char **argv, char *inbuf)
 		return;
 	}
 
-	do_read(fd, inbuf, sizeof(inbuf));
+	do_read(fd, inbuf, GROUPD_DUMP_SIZE);
 
 	close(fd);
 }
 
-static int do_log(char *comment)
-{
-	char buf[GROUPD_MSGLEN];
-	int fd, rv;
-
-	fd = connect_daemon(GROUPD_SOCK_PATH);
-	if (fd < 0)
-		return fd;
-	memset(buf, 0, sizeof(buf));
-	snprintf(buf, sizeof(buf), "log %s", comment);
-	rv = write(fd, &buf, GROUPD_MSGLEN);
-	close(fd);
-	return rv;
-}
-
 int main(int argc, char **argv)
 {
-	int rv, version;
+	int rv, version = 0; 
 
 	prog_name = argv[0];
 	decode_arguments(argc, argv);
 
 	switch (operation) {
+	case OP_COMPAT:
+		rv = group_get_version(&version);
+		if (rv < 0)
+			version = -1;
+
+		switch (version) {
+		case -1:
+			break;
+		case -EAGAIN:
+			printf("groupd compatibility mode 2 (pending)\n");
+			break;
+		case GROUP_LIBGROUP:
+			printf("groupd compatibility mode 1\n");
+			break;
+		case GROUP_LIBCPG:
+			printf("groupd compatibility mode 0\n");
+			break;
+		default:
+			printf("groupd compatibility mode %d\n", version);
+			break;
+		}
+
+		if (rv < 0)
+			exit(EXIT_FAILURE);
+		break;
+
 	case OP_LIST:
-		if (all_daemons) {
+
+		rv = group_get_version(&version);
+		if (rv < 0)
+			version = -1;
+
+		switch (version) {
+		case -1:
+			printf("groupd not running\n");
+			version = GROUP_LIBCPG;
+			break;
+		case -EAGAIN:
+			printf("groupd compatibility mode 2 (pending)\n");
+			break;
+		case GROUP_LIBGROUP:
+			printf("groupd compatibility mode 1\n");
+			break;
+		case GROUP_LIBCPG:
+			printf("groupd compatibility mode 0\n");
+			break;
+		default:
+			printf("groupd compatibility mode %d\n", version);
+			break;
+		}
+
+		if ((cfg_groupd_compat == 0) ||
+		    (cfg_groupd_compat == 2 && version == GROUP_LIBCPG)) {
+			/* show the new cluster3 data (from daemons) in
+			   the new daemon-specific format */
+
 			if (verbose || ls_all_nodes) {
 				system("fence_tool ls -n");
 				system("dlm_tool ls -n");
@@ -625,20 +679,31 @@ int main(int argc, char **argv)
 				system("dlm_tool ls");
 				system("gfs_control ls");
 			}
-		} else {
-			rv = group_get_version(&version);
 
-			if (version == -EAGAIN) {
-				printf("groupd detecting version...\n");
-			} else if (!rv && version == GROUP_LIBGROUP) {
-				groupd_list(argc, argv);
-			} else {
-				fenced_list();
-				dlm_controld_list();
-				gfs_controld_list();
-			}
+			if (version == GROUP_LIBGROUP)
+				printf("Run 'group_tool ls -g1' for groupd information.\n");
+			break;
 		}
-		break;
+
+		if ((cfg_groupd_compat == 1) ||
+		    (cfg_groupd_compat == 2 && version == GROUP_LIBGROUP)) {
+			/* show the same old cluster2 data (from groupd)
+			   in the same old format as cluster2 */
+
+			groupd_list(argc, argv);
+
+			if (version == GROUP_LIBCPG)
+				printf("Run 'group_tool ls -g0' for daemon information.\n");
+			break;
+		}
+				
+#if 0
+		/* do we want to add an option that will use these functions
+		   to "fake" new cluster3 data in the old cluster2 format? */
+		fenced_list();
+		dlm_controld_list();
+		gfs_controld_list();
+#endif
 
 	case OP_DUMP:
 		if (opt_ind && opt_ind < argc) {
@@ -699,11 +764,6 @@ int main(int argc, char **argv)
 		}
 
 		break;
-
-	case OP_LOG:
-		if (opt_ind && opt_ind < argc) {
-			return do_log(argv[opt_ind]);
-		}
 	}
 
 	return 0;
