@@ -13,6 +13,8 @@
 #include <assert.h>
 #include <event.h>
 #include <sets.h>
+#include <fo_domain.h>
+#include <groups.h>
 
 /* Use address field in this because we never use it internally,
    and there is no extra space in the cman_node_t type.
@@ -39,11 +41,8 @@ pthread_mutex_t status_mutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_rwlock_t resource_lock = PTHREAD_RWLOCK_INITIALIZER;
 
 void res_build_name(char *, size_t, resource_t *);
-int get_rg_state_local(char *, rg_state_t *);
-int group_migratory(char *groupname, int lock);
-int _group_property(char *groupname, char *property, char *ret, size_t len);
-int restart_threshold_exceeded(restart_counter_t arg);
-
+static int _group_property(const char *groupname, const char *property,
+			   char *ret, size_t len);
 
 struct status_arg {
 	msgctx_t *ctx;
@@ -58,7 +57,7 @@ struct status_arg {
  */
 int
 node_should_start_safe(uint32_t nodeid, cluster_member_list_t *membership,
-		       char *rg_name)
+		       const char *rg_name)
 {
 	int ret;
 
@@ -107,14 +106,14 @@ count_resource_groups(cluster_member_list_t *ml)
 		res_build_name(rgname, sizeof(rgname), res);
 
 		if (rg_lock(rgname, &lockp) < 0) {
-			log_printf(LOG_ERR, "#XX: Unable to obtain cluster "
+			logt_print(LOG_ERR, "#XX: Unable to obtain cluster "
 			       "lock @ %s:%d: %s\n", __FILE__, __LINE__,
 			       strerror(errno));
 			continue;
 		}
 
 		if (get_rg_state(rgname, &st) < 0) {
-			log_printf(LOG_ERR, "#34: Cannot get status "
+			logt_print(LOG_ERR, "#34: Cannot get status "
 			       "for service %s\n", rgname);
 			rg_unlock(&lockp);
 			continue;
@@ -132,7 +131,7 @@ count_resource_groups(cluster_member_list_t *ml)
 
 		++mp->cn_svccount;
 
-		val = res_attr_value(res, "exclusive");
+		val = (char *)res_attr_value(res, "exclusive");
 		if (val && ((!strcmp(val, "yes") ||
 				     (atoi(val)>0))) ) {
 			++mp->cn_svcexcl;
@@ -148,7 +147,7 @@ count_resource_groups(cluster_member_list_t *ml)
 static inline int 
 is_exclusive_res(resource_t *res)
 {
-	char *val;
+	const char *val;
 
 	val = res_attr_value(res, "exclusive");
 	if (val && ((!strcmp(val, "yes") ||
@@ -161,7 +160,7 @@ is_exclusive_res(resource_t *res)
 
 /* Locked exported function */
 int 
-is_exclusive(char *svcName)
+is_exclusive(const char *svcName)
 {
 	int ret = 0;
 	resource_t *res = NULL;
@@ -179,8 +178,8 @@ is_exclusive(char *svcName)
 }
 
 
-resource_node_t *
-node_by_ref(resource_node_t **tree, char *name)
+static resource_node_t *
+node_by_ref(resource_node_t **tree, const char *name)
 {
 	resource_t *res;
 	resource_node_t *node, *ret = NULL;
@@ -266,7 +265,8 @@ have_exclusive_resources(void)
 
 
 int
-check_exclusive_resources(cluster_member_list_t *membership, char *svcName)
+check_exclusive_resources(cluster_member_list_t *membership,
+			  const char *svcName)
 {
 	cman_node_t *mp;
 	int exclusive, count, excl; 
@@ -302,7 +302,7 @@ check_exclusive_resources(cluster_member_list_t *membership, char *svcName)
  */
 uint32_t
 best_target_node(cluster_member_list_t *allowed, uint32_t owner,
-		 char *rg_name, int lock)
+		 const char *rg_name, int lock)
 {
 	int x;
 	int highscore = 1;
@@ -342,7 +342,7 @@ best_target_node(cluster_member_list_t *allowed, uint32_t owner,
 		   it's an exclusive service and the target node already
 		   is running a service. */
 		res = find_root_by_ref(&_resources, rg_name);
-		val = res_attr_value(res, "exclusive");
+		val = (char *)res_attr_value(res, "exclusive");
 		exclusive = val && ((!strcmp(val, "yes") || (atoi(val)>0)));
 
 		if (lock)
@@ -376,7 +376,7 @@ best_target_node(cluster_member_list_t *allowed, uint32_t owner,
 int
 check_depend(resource_t *res)
 {
-	char *val;
+	const char *val;
 	rg_state_t rs;
 
 	val = res_attr_value(res, "depend");
@@ -392,7 +392,7 @@ check_depend(resource_t *res)
 
 
 int
-check_depend_safe(char *rg_name)
+check_depend_safe(const char *rg_name)
 {
 	resource_t *res;
 	int ret;
@@ -409,8 +409,8 @@ check_depend_safe(char *rg_name)
 }
 
 
-int
-check_rdomain_crash(char *svcName)
+static int
+check_rdomain_crash(const char *svcName)
 {
 	int *nodes = NULL, nodecount = 0;
 	int *fd_nodes = NULL, fd_nodecount = 0, fl = 0;
@@ -420,6 +420,8 @@ check_rdomain_crash(char *svcName)
 	if (_group_property(svcName, "domain", fd_name, sizeof(fd_name)) != 0)
 		goto out_free;
 
+	member_online_set(&nodes, &nodecount);
+
 	if (node_domain_set(&_domains, fd_name, &fd_nodes,
 			    &fd_nodecount, &fl) != 0)
 		goto out_free;
@@ -432,7 +434,7 @@ check_rdomain_crash(char *svcName)
 		goto out_free;
 
 	if (icount == 0) {
-		log_printf(LOG_NOTICE, "Marking %s as stopped: "
+		logt_print(LOG_NOTICE, "Marking %s as stopped: "
 		       "Restricted domain unavailable\n", svcName);
 		rt_enqueue_request(svcName, RG_STOP, NULL, 0, 0,
 				   0, 0);
@@ -455,7 +457,7 @@ out_free:
   If it is running and we're a better member to run it, then ask for
   it.
  */
-void
+static void
 consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	       cluster_member_list_t *membership)
 {
@@ -486,12 +488,12 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	   autostart is disabled.  If it is, leave it stopped */
 	if (svcStatus->rs_state == RG_STATE_STOPPED &&
 		    svcStatus->rs_transition == 0) {
-		val = res_attr_value(node->rn_resource, "autostart");
+		val = (char *)res_attr_value(node->rn_resource, "autostart");
 		autostart = !(val && ((!strcmp(val, "no") ||
 				     (atoi(val)==0))));
 		if (!autostart) {
 			/*
-			log_printf(LOG_DEBUG,
+			logt_print(LOG_DEBUG,
 			       "Skipping RG %s: Autostart disabled\n",
 			       svcName);
 			 */
@@ -500,14 +502,14 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 			   confusion!
 			 */
 			if (rg_lock(svcName, &lockp) < 0) {
-				log_printf(LOG_ERR, "#XX: Unable to obtain cluster "
+				logt_print(LOG_ERR, "#XX: Unable to obtain cluster "
 				       "lock @ %s:%d: %s\n", __FILE__, __LINE__,
 				       strerror(errno));
 				return;
 			}
 
 			if (get_rg_state(svcName, svcStatus) != 0) {
-				log_printf(LOG_ERR, "#34: Cannot get status "
+				logt_print(LOG_ERR, "#34: Cannot get status "
 				       "for service %s\n", svcName);
 				rg_unlock(&lockp);
 				return;
@@ -528,16 +530,16 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	/* See if service this one depends on is running.  If not,
            don't start it */
 	if (check_depend(node->rn_resource) == 0) {
-		log_printf(LOG_DEBUG,
+		logt_print(LOG_DEBUG,
 		       "Skipping RG %s: Dependency missing\n", svcName);
 		return;
 	}
 
-	val = res_attr_value(node->rn_resource, "exclusive");
+	val = (char *)res_attr_value(node->rn_resource, "exclusive");
 	exclusive = val && ((!strcmp(val, "yes") || (atoi(val)>0)));
 
 	if (exclusive && mp->cn_svccount) {
-		log_printf(LOG_DEBUG,
+		logt_print(LOG_DEBUG,
 		       "Skipping RG %s: Exclusive and I am running services\n",
 		       svcName);
 		return;
@@ -548,7 +550,7 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	   service.
 	 */
 	if (mp->cn_svcexcl) {
-		log_printf(LOG_DEBUG,
+		logt_print(LOG_DEBUG,
 		       "Skipping RG %s: I am running an exclusive service\n",
 		       svcName);
 		return;
@@ -568,8 +570,8 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 }
 
 
-void
-consider_relocate(char *svcName, rg_state_t *svcStatus, uint32_t nodeid,
+static void
+consider_relocate(const char *svcName, rg_state_t *svcStatus, uint32_t nodeid,
 		  cluster_member_list_t *membership)
 {
 	int a, b, req = RG_RELOCATE;
@@ -603,7 +605,7 @@ consider_relocate(char *svcName, rg_state_t *svcStatus, uint32_t nodeid,
 		req = RG_MIGRATE;
 	}
 
-	log_printf(LOG_NOTICE, "%s %s to better node %s\n",
+	logt_print(LOG_NOTICE, "%s %s to better node %s\n",
 	       req==RG_MIGRATE ? "Migrating":"Relocating",
 	       svcName,
 	       memb_id_to_name(membership, nodeid));
@@ -684,7 +686,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
 	int ret;
 
 	if (rg_locked()) {
-		log_printf(LOG_DEBUG,
+		logt_print(LOG_DEBUG,
 			"Resource groups locked; not evaluating\n");
 		return -EAGAIN;
 	}
@@ -705,7 +707,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
 		 * status.
 		 */
 		if ((ret = rg_lock(svcName, &lockp)) < 0) {
-			log_printf(LOG_ERR,
+			logt_print(LOG_ERR,
 			       "#33: Unable to obtain cluster lock: %s\n",
 			       strerror(-ret));
 			pthread_rwlock_unlock(&resource_lock);
@@ -714,7 +716,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
 		}
 		
 		if (get_rg_state(svcName, &svcStatus) != 0) {
-			log_printf(LOG_ERR,
+			logt_print(LOG_ERR,
 			       "#34: Cannot get status for service %s\n",
 			       svcName);
 			rg_unlock(&lockp);
@@ -724,7 +726,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
 		rg_unlock(&lockp);
 
 		if (svcStatus.rs_owner == 0)
-			nodeName = "none";
+			nodeName = (char *)"none";
 		else
 			nodeName = memb_id_to_name(membership,
 						   svcStatus.rs_owner);
@@ -736,7 +738,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
 			continue;
 		}
 
-		log_printf(LOG_DEBUG, "Evaluating RG %s, state %s, owner "
+		logt_print(LOG_DEBUG, "Evaluating RG %s, state %s, owner "
 		       "%s\n", svcName,
 		       rg_state_str(svcStatus.rs_state),
 		       nodeName);
@@ -771,7 +773,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
 	pthread_rwlock_unlock(&resource_lock);
 	free_member_list(membership);
 
-	log_printf(LOG_DEBUG, "Event (%d:%d:%d) Processed\n", local,
+	logt_print(LOG_DEBUG, "Event (%d:%d:%d) Processed\n", local,
 	       (int)nodeid, nodeStatus);
 
 	return 0;
@@ -784,7 +786,7 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
  * @see			eval_groups
  */
 int
-group_event(char __attribute__ ((unused)) *rg_name,
+group_event(const char __attribute__ ((unused)) *rg_name,
 	    uint32_t state,
 	    int __attribute__ ((unused)) owner)
 {
@@ -795,7 +797,7 @@ group_event(char __attribute__ ((unused)) *rg_name,
 	int depend;
 
 	if (rg_locked()) {
-		log_printf(LOG_DEBUG,
+		logt_print(LOG_DEBUG,
 			"Resource groups locked; not evaluating\n");
 		return -EAGAIN;
 	}
@@ -817,7 +819,7 @@ group_event(char __attribute__ ((unused)) *rg_name,
 			continue;
 
 		if (svcStatus.rs_owner == 0)
-			nodeName = "none";
+			nodeName = (char *)"none";
 		else
 			nodeName = memb_id_to_name(membership,
 						   svcStatus.rs_owner);
@@ -847,7 +849,7 @@ group_event(char __attribute__ ((unused)) *rg_name,
 		    svcStatus.rs_state == RG_STATE_STOPPED &&
 		    state == RG_STATE_STARTED) {
 
-			log_printf(LOG_DEBUG, "Evaluating RG %s, state %s, owner "
+			logt_print(LOG_DEBUG, "Evaluating RG %s, state %s, owner "
 			       "%s\n", svcName,
 			       rg_state_str(svcStatus.rs_state),
 			       nodeName);
@@ -863,7 +865,7 @@ group_event(char __attribute__ ((unused)) *rg_name,
 		    svcStatus.rs_state == RG_STATE_STARTED &&
 		    svcStatus.rs_owner == (uint32_t)my_id()) {
 
-			log_printf(LOG_WARNING, "Stopping service %s: Dependency missing\n",
+			logt_print(LOG_WARNING, "Stopping service %s: Dependency missing\n",
 			       svcName);
 			rt_enqueue_request(svcName, RG_STOP, NULL, 0, my_id(),
 					   0, 0);
@@ -882,7 +884,7 @@ group_event(char __attribute__ ((unused)) *rg_name,
   Tells us if a resource group can be migrated.
  */
 int
-group_migratory(char *groupname, int lock)
+group_migratory(const char *groupname, int lock)
 {
 	resource_node_t *rn;
 	resource_t *res;
@@ -941,7 +943,7 @@ out_unlock:
    @return 		0 on success, 1 on failure/error.
  */
 int
-group_op(char *groupname, int op)
+group_op(const char *groupname, int op)
 {
 	resource_t *res;
 	int ret = -1;
@@ -964,6 +966,9 @@ group_op(char *groupname, int op)
 	case RG_STATUS:
 		ret = res_status(&_tree, res, NULL);
 		break;
+	case RG_STATUS_INQUIRY:
+		ret = res_status_inquiry(&_tree, res, NULL);
+		break;
 	case RG_CONDSTOP:
 		ret = res_condstop(&_tree, res, NULL);
 		break;
@@ -998,7 +1003,7 @@ group_op(char *groupname, int op)
 
 
 int
-group_migrate(char *groupname, int target)
+group_migrate(const char *groupname, int target)
 {
 	resource_node_t *rn = NULL, *tmp;
 	resource_t *res;
@@ -1007,7 +1012,7 @@ group_migrate(char *groupname, int target)
 	cluster_member_list_t *membership;
 
 	if (target <= 0) {
-		log_printf(LOG_WARNING,
+		logt_print(LOG_WARNING,
 		       "Illegal node ID %d during migrate operation\n",
 		       target);
 		return RG_EINVAL;
@@ -1015,7 +1020,7 @@ group_migrate(char *groupname, int target)
 
 	membership = member_list();
 	if (!membership) {
-		log_printf(LOG_ERR, "Unable to determine membership during "
+		logt_print(LOG_ERR, "Unable to determine membership during "
 		       "migrate operation\n");
 		return RG_EFAIL;
 	}
@@ -1024,7 +1029,7 @@ group_migrate(char *groupname, int target)
 	
 	tgt_name = memb_id_to_name(membership, target);
 	if (!tgt_name) {
-		log_printf(LOG_WARNING, "Node ID %d not in membership during "
+		logt_print(LOG_WARNING, "Node ID %d not in membership during "
 		       "migrate operation\n", target);
 		ret = RG_EINVAL;
 		goto out;
@@ -1032,7 +1037,7 @@ group_migrate(char *groupname, int target)
 
 	res = find_root_by_ref(&_resources, groupname);
 	if (!res) {
-		log_printf(LOG_WARNING,
+		logt_print(LOG_WARNING,
 		       "Unable to find '%s' in resource list during"
 		       "migrate operation\n", groupname);
 		goto out;
@@ -1046,20 +1051,20 @@ group_migrate(char *groupname, int target)
 	} while (!list_done(&_tree, tmp));
 
 	if (!rn) {
-		log_printf(LOG_WARNING,
+		logt_print(LOG_WARNING,
 		       "Unable to find '%s' it top level of resource "
 		       "tree during migrate operation\n", groupname);
 		goto out;
 	}
 	
-	log_printf(LOG_NOTICE, "Migrating %s to %s\n", groupname, tgt_name);
+	logt_print(LOG_NOTICE, "Migrating %s to %s\n", groupname, tgt_name);
 	ret = res_exec(rn, RS_MIGRATE, tgt_name, 0);
 	if (ret == 0) {
-		log_printf(LOG_NOTICE,
+		logt_print(LOG_NOTICE,
 		       "Migration of %s to %s completed\n",
 		       groupname, tgt_name);
 	} else {
-		log_printf(LOG_ERR, 
+		logt_print(LOG_ERR, 
 		       "Migration of %s to %s failed; return code %d\n",
 		       groupname, tgt_name, ret);
 	}
@@ -1080,8 +1085,9 @@ out:
    @param len		Length of buffer pointed to by ret
    @return		0 on success, -1 on failure.
  */
-int
-_group_property(char *groupname, char *property, char *ret, size_t len)
+static int
+_group_property(const char *groupname, const char *property,
+		char *ret, size_t len)
 {
 	resource_t *res = NULL;
 	int x = 0;
@@ -1103,7 +1109,8 @@ _group_property(char *groupname, char *property, char *ret, size_t len)
 
 
 int
-group_property(char *groupname, char *property, char *ret_val, size_t len)
+group_property(const char *groupname, const char *property,
+	       char *ret_val, size_t len)
 {
 	int ret = -1;
 	pthread_rwlock_rdlock(&resource_lock);
@@ -1120,7 +1127,7 @@ group_property(char *groupname, char *property, char *ret_val, size_t len)
   @param rgname		Resource group name whose state we want to send.
   @see send_rg_states
  */
-void
+static void
 send_rg_state(msgctx_t *ctx, char *rgname, int fast)
 {
 	rg_state_msg_t msg, *msgp = &msg;
@@ -1243,7 +1250,7 @@ send_rg_states(msgctx_t *ctx, int fast)
 
 
 int
-svc_exists(char *svcname)
+svc_exists(const char *svcname)
 {
 	resource_node_t *node;
 	int ret = 0;
@@ -1267,7 +1274,8 @@ svc_exists(char *svcname)
 
 
 void
-rg_doall(int request, int block, char __attribute__ ((unused)) *debugfmt)
+rg_doall(int request, int block,
+	 const char __attribute__ ((unused)) *debugfmt)
 {
 	resource_node_t *curr;
 	rg_state_t svcblk;
@@ -1280,7 +1288,7 @@ rg_doall(int request, int block, char __attribute__ ((unused)) *debugfmt)
 		res_build_name(rg, sizeof(rg), curr->rn_resource);
 
 		//if (debugfmt)
-			//log_printf(LOG_DEBUG, debugfmt, rg);
+			//logt_print(LOG_DEBUG, debugfmt, rg);
 
 		/* Optimization: Don't bother even queueing the request
 		   during the exit case if we don't own it */
@@ -1311,7 +1319,7 @@ rg_doall(int request, int block, char __attribute__ ((unused)) *debugfmt)
 /**
   Stop changed resources.
  */
-void *
+static void *
 q_status_checks(void __attribute__ ((unused)) *arg)
 {
 	resource_node_t *curr;
@@ -1376,7 +1384,7 @@ do_status_checks(void)
 /**
   Stop changed resources.
  */
-void
+static void
 do_condstops(void)
 {
 	resource_node_t *curr;
@@ -1385,7 +1393,7 @@ do_condstops(void)
 	int need_kill;
 	char rg[64];
 
-	log_printf(LOG_INFO, "Stopping changed resources.\n");
+	logt_print(LOG_INFO, "Stopping changed resources.\n");
 
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
@@ -1405,14 +1413,14 @@ do_condstops(void)
 		need_kill = 0;
 		if (curr->rn_resource->r_flags & RF_NEEDSTOP) {
 			need_kill = 1;
-			log_printf(LOG_DEBUG, "Removing %s\n", rg);
+			logt_print(LOG_DEBUG, "Removing %s\n", rg);
 		}
 
 		if (!curr->rn_child && ((curr->rn_resource->r_rule->rr_flags &
 		    RF_DESTROY) == 0) && group_migratory(rg, 0) &&
 		    need_kill == 1) {
 			/* Do something smart here: flip state? */
-			log_printf(LOG_NOTICE,
+			logt_print(LOG_NOTICE,
 			       "%s removed from the config, but I am not stopping it.\n",
 			       rg);
 			if (rg_lock(rg, &lockp) != 0)
@@ -1439,7 +1447,7 @@ cont:
 /**
   Start changed resources.
  */
-void
+static void
 do_condstarts(void)
 {
 	resource_node_t *curr;
@@ -1448,7 +1456,7 @@ do_condstarts(void)
 	int need_init, new_groups = 0, autostart;
 	struct dlm_lksb lockp;
 
-	log_printf(LOG_INFO, "Starting changed resources.\n");
+	logt_print(LOG_INFO, "Starting changed resources.\n");
 
 	/* Pass 1: Start any normally changed resources */
 	pthread_rwlock_rdlock(&resource_lock);
@@ -1483,14 +1491,14 @@ do_condstarts(void)
 
 		if (need_init) {
 			++new_groups;
-			log_printf(LOG_NOTICE, "Initializing %s\n", rg);
+			logt_print(LOG_NOTICE, "Initializing %s\n", rg);
 		}
 
 		if (!curr->rn_child && ((curr->rn_resource->r_rule->rr_flags &
 		    RF_INIT) == 0) && group_migratory(rg, 0) &&
 		    need_init == 1) {
 			/* Do something smart here? */
-			log_printf(LOG_NOTICE,
+			logt_print(LOG_NOTICE,
 			       "%s was added to the config, but I am not initializing it.\n",
 			       rg);
 			continue;
@@ -1537,7 +1545,7 @@ do_condstarts(void)
 		}
 
 		/* Set it up for an auto-start */
-		val = res_attr_value(curr->rn_resource, "autostart");
+		val = (char *)res_attr_value(curr->rn_resource, "autostart");
 		autostart = !(val && ((!strcmp(val, "no") ||
 				     (atoi(val)==0))));
 		if (autostart)
@@ -1557,36 +1565,6 @@ do_condstarts(void)
 }
 
 
-int
-check_config_update(int *new, int *old)
-{
-	int newver = 0, fd, ret = 0;
-	char *val = NULL;
-
-       	fd = ccs_lock();
-	if (fd == -1) {
-		return 0;
-	}
-
-	if (ccs_get(fd, "/cluster/@config_version", &val) == 0) {
-		newver = atoi(val);
-	}
-
-	if (val)
-		free(val);
-
-	pthread_mutex_lock(&config_mutex);
-	if (newver && newver != config_version)
-		ret = 1;
-	if (new) *new = newver;
-	if (old) *old = config_version;
-	pthread_mutex_unlock(&config_mutex);
-	ccs_unlock(fd);
-
-	return ret;
-}
-
-
 void
 dump_config_version(FILE *fp)
 {
@@ -1594,6 +1572,30 @@ dump_config_version(FILE *fp)
 }
 
 
+/**
+ Copy out the incarnations after doing CONDSTOPs
+ */
+static int
+copy_incarnations(resource_t **leftres, resource_t **rightres)
+{
+	resource_t *lc, *rc;
+
+	list_do(leftres, lc) {
+		rc = find_resource_by_ref(rightres, lc->r_rule->rr_type,
+					  primary_attr_value(lc));
+		/* Resource does not exist */
+		if (!rc)
+			continue;
+
+		/* Ok, see if the resource is the same */
+		if (!rescmp(lc, rc))
+			rc->r_incarnations = lc->r_incarnations;
+	} while (!list_done(leftres, lc));
+
+	return 0;
+}
+
+
 /**
   Initialize resource groups.  This reads all the resource groups from 
   CCS, builds the tree, etc.  Ideally, we'll have a similar function 
@@ -1614,19 +1616,20 @@ init_resource_groups(int reconfigure, int do_init)
 	char *val;
 
 	if (reconfigure)
-		log_printf(LOG_NOTICE, "Reconfiguring\n");
-	log_printf(LOG_INFO, "Loading Service Data\n");
-	log_printf(LOG_DEBUG, "Loading Resource Rules\n");
+		logt_print(LOG_NOTICE, "Reconfiguring\n");
+
+	logt_print(LOG_INFO, "Loading Service Data\n");
+	logt_print(LOG_DEBUG, "Loading Resource Rules\n");
 	if (load_resource_rules(RESOURCE_ROOTDIR, &rulelist) != 0) {
 		return -1;
 	}
 	x = 0;
 	list_do(&rulelist, rule) { ++x; } while (!list_done(&rulelist, rule));
-	log_printf(LOG_DEBUG, "%d rules loaded\n", x);
+	logt_print(LOG_DEBUG, "%d rules loaded\n", x);
 
        	fd = ccs_lock();
 	if (fd == -1) {
-		log_printf(LOG_CRIT, "#5: Couldn't connect to ccsd!\n");
+		logt_print(LOG_CRIT, "#5: Couldn't connect to ccsd!\n");
 		return -1;
 	}
 
@@ -1643,10 +1646,16 @@ init_resource_groups(int reconfigure, int do_init)
 		free(val);
 	}
 
-	log_printf(LOG_DEBUG, "Building Resource Trees\n");
+	/* Wait for any pending requests */
+	rg_wait_threads();
+	/* Block operations that would break during configuration
+	   changes */
+	rg_clear_initialized(FL_CONFIG);
+
+	logt_print(LOG_DEBUG, "Building Resource Trees\n");
 	/* About to update the entire resource tree... */
 	if (load_resources(fd, &reslist, &rulelist) != 0) {
-		log_printf(LOG_CRIT, "#6: Error loading services\n");
+		logt_print(LOG_CRIT, "#6: Error loading services\n");
 		destroy_resources(&reslist);
 		destroy_resource_rules(&rulelist);
 		ccs_unlock(fd);
@@ -1654,7 +1663,7 @@ init_resource_groups(int reconfigure, int do_init)
 	}
 
 	if (build_resource_tree(fd, &tree, &rulelist, &reslist) != 0) {
-		log_printf(LOG_CRIT, "#7: Error building resource tree\n");
+		logt_print(LOG_CRIT, "#7: Error building resource tree\n");
 		destroy_resource_tree(&tree);
 		destroy_resources(&reslist);
 		destroy_resource_rules(&rulelist);
@@ -1664,13 +1673,15 @@ init_resource_groups(int reconfigure, int do_init)
 
 	x = 0;
 	list_do(&reslist, res) { ++x; } while (!list_done(&reslist, res));
-	log_printf(LOG_DEBUG, "%d resources defined\n", x);
+	logt_print(LOG_DEBUG, "%d resources defined\n", x);
 
-	log_printf(LOG_DEBUG, "Loading Failover Domains\n");
+	logt_print(LOG_DEBUG, "Loading Failover Domains\n");
 	construct_domains(fd, &domains);
 	x = 0;
 	list_do(&domains, fod) { ++x; } while (!list_done(&domains, fod));
-	log_printf(LOG_DEBUG, "%d domains defined\n", x);
+	logt_print(LOG_DEBUG, "%d domains defined\n", x);
+
+	logt_print(LOG_DEBUG, "Loading Event Triggers\n");
 	construct_events(fd, &evt);
 	cnt = 0;
 	if (evt) {
@@ -1687,7 +1698,7 @@ init_resource_groups(int reconfigure, int do_init)
 			cnt += y;
 		}
 	}
-	log_printf(LOG_DEBUG, "%d events defined\n", x);
+	logt_print(LOG_DEBUG, "%d events defined\n", cnt);
 	
 
 	/* Reconfiguration done */
@@ -1701,6 +1712,8 @@ init_resource_groups(int reconfigure, int do_init)
 		pthread_rwlock_unlock(&resource_lock);
 
 		do_condstops();
+
+		copy_incarnations(&_resources, &reslist);
 	}
 
 	/* Swap in the new configuration */
@@ -1725,29 +1738,30 @@ init_resource_groups(int reconfigure, int do_init)
 	if (reconfigure) {
 		/* Switch to read lock and do the up-half of the
 		   reconfig request */
-		log_printf(LOG_INFO, "Restarting changed resources.\n");
+		logt_print(LOG_INFO, "Restarting changed resources.\n");
 		do_condstarts();
 	} else {
 		if (do_init) {
 			/* Do initial stop-before-start */
-			log_printf(LOG_INFO, "Initializing Services\n");
+			logt_print(LOG_INFO, "Initializing Services\n");
 			rg_doall(RG_INIT, 1, "Initializing %s\n");
-			log_printf(LOG_INFO, "Services Initialized\n");
+			logt_print(LOG_INFO, "Services Initialized\n");
 		} else {
-			log_printf(LOG_INFO, "Skipping stop-before-start: overridden by administrator\n");
+			logt_print(LOG_INFO, "Skipping stop-before-start: overridden by administrator\n");
 		}
-		rg_set_initialized();
+		rg_set_initialized(FL_INIT);
 	}
+	rg_set_initialized(FL_CONFIG);
 
 	return 0;
 }
 
 
 void
-get_recovery_policy(char *rg_name, char *buf, size_t buflen)
+get_recovery_policy(const char *rg_name, char *buf, size_t buflen)
 {
 	resource_t *res;
-	char *val;
+	const char *val;
 
 	pthread_rwlock_rdlock(&resource_lock);
 
@@ -1765,11 +1779,12 @@ get_recovery_policy(char *rg_name, char *buf, size_t buflen)
 
 
 int
-get_service_property(char *rg_name, char *prop, char *buf, size_t buflen)
+get_service_property(const char *rg_name, const char *prop,
+		     char *buf, size_t buflen)
 {
 	int ret = 0;
 	resource_t *res;
-	char *val;
+	const char *val;
 
 	memset(buf, 0, buflen);
 
@@ -1819,7 +1834,7 @@ get_service_property(char *rg_name, char *prop, char *buf, size_t buflen)
 
 
 int
-check_restart(char *rg_name)
+add_restart(const char *rg_name)
 {
 	resource_node_t *node;
 	int ret = 1;
@@ -1828,11 +1843,23 @@ check_restart(char *rg_name)
 	node = node_by_ref(&_tree, rg_name);
 	if (node) {
 		ret = restart_add(node->rn_restart_counter);
-		if (ret) {
-			/* Clear it out - caller is about 
-			   to relocate the service anyway */
-			restart_clear(node->rn_restart_counter);
-		}
+	}
+	pthread_rwlock_unlock(&resource_lock);
+
+	return ret;
+}
+
+
+int
+check_restart(const char *rg_name)
+{
+	resource_node_t *node;
+	int ret = 0;
+
+	pthread_rwlock_rdlock(&resource_lock);
+	node = node_by_ref(&_tree, rg_name);
+	if (node) {
+		ret = restart_threshold_exceeded(node->rn_restart_counter);
 	}
 	pthread_rwlock_unlock(&resource_lock);
 
