@@ -1,3 +1,7 @@
+#include <time.h>
+#include <list.h>
+#include <restart_counter.h>
+#include <reslist.h>
 #include <message.h>
 #include <resgroup.h>
 #include <rg_locks.h>
@@ -34,9 +38,7 @@ static pthread_mutex_t reslist_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static resthread_t *find_resthread_byname(const char *resgroupname);
 static int spawn_if_needed(const char *resgroupname);
-int rt_enqueue_request(const char *resgroupname, int request,
-		       msgctx_t *response_ctx, int max, uint32_t target,
-		       int arg0, int arg1);
+
 int central_events_enabled(void);
 
 
@@ -59,7 +61,7 @@ dump_threads(FILE *fp)
 			rt,
 			rg_req_str(rt->rt_request),
 			rt->rt_request);
-		if (!*rt->rt_queue) {
+		if (rt->rt_queue) {
 			fprintf(fp, "    Pending requests: \n");
 			list_for(rt->rt_queue, req, y) {
 				fprintf(fp, "      %s tgt:%d  ctx:%p  a0:%d  a1:%d\n",
@@ -582,6 +584,8 @@ spawn_if_needed(const char *resgroupname)
 	int ret;
 	resthread_t *resgroup = NULL;
 
+retry:
+	resgroup = NULL;
 	pthread_mutex_lock(&reslist_mutex);
 	while (resgroup == NULL) {
 		resgroup = find_resthread_byname(resgroupname);
@@ -599,8 +603,8 @@ spawn_if_needed(const char *resgroupname)
 	ret = (resgroup->rt_status == RG_STATE_STOPPING);
 
 	pthread_mutex_unlock(&reslist_mutex);
-	if (wait_initialize(resgroupname) < 0) {
-		return -1;
+	if (!ret && wait_initialize(resgroupname) < 0) {
+		goto retry;
 	}
 
 	return ret;
