@@ -563,6 +563,100 @@ do_status()
 }
 
 
+#
+# virsh "path" attribute support
+#
+check_config_file()
+{
+	declare path=$1
+
+	if [ -f "$path/$OCF_RESKEY_name" ]; then
+		echo $path/$OCF_RESKEY_name
+		return 2
+	elif [ -f "$path/$OCF_RESKEY_name.xml" ]; then
+		echo $path/$OCF_RESKEY_name.xml
+		return 2
+	fi
+
+	return 0
+}
+
+
+parse_input()
+{
+	declare delim=$1
+	declare input=$2
+	declare func=$3
+	declare inp
+	declare value
+
+	while [ -n "$input" ]; do
+		value=${input/$delim*/}
+		if [ -n "$value" ]; then
+			eval $func $value
+			if [ $? -eq 2 ]; then
+				return 0
+			fi
+		fi
+		inp=${input/$value$delim/}
+		if [ "$input" = "$inp" ]; then
+			inp=${input/$value/}
+		fi
+		input=$inp
+	done
+}
+
+
+search_config_path()
+{
+	declare config_file=$(parse_input ":" "$OCF_RESKEY_path" check_config_file)
+
+	if [ -n "$config_file" ]; then
+		export OCF_RESKEY_xmlfile=$config_file
+		return 0
+	fi
+
+	return 1
+}
+
+
+choose_management_tool()
+{
+	declare -i is_xml
+
+	#
+	# Don't override user value for use_virsh if one is given
+	#
+	if [ -n "$OCF_RESKEY_use_virsh" ]; then
+		return 0
+	fi
+
+	which xmllint &> /dev/null
+	if [ $? -ne 0 ]; then
+		ocf_log warning "Could not find xmllint; assuming virsh mode"
+		export OCF_RESKEY_use_virsh=1
+		unset OCF_RESKEY_path
+		return 0
+	fi
+
+	xmllint $OCF_RESKEY_xmlfile &> /dev/null
+	is_xml=$?
+
+	if [ $is_xml -eq 0 ]; then
+		ocf_log debug "$OCF_RESKEY_xmlfile is XML; using virsh"
+		export OCF_RESKEY_use_virsh=1
+		unset OCF_RESKEY_path
+	else
+		ocf_log debug "$OCF_RESKEY_xmlfile is not XML; using xm"
+		export OCF_RESKEY_use_virsh=0
+		unset OCF_RESKEY_xmlfile
+	fi
+
+	return 0
+}
+
+
+
 validate_all()
 {
 	if [ "$(id -u)" != "0" ]; then
@@ -592,46 +686,50 @@ validate_all()
 			ocf_log err "Cannot use $OCF_RESKEY_hypervisor hypervisor without using virsh"
 			return $OCF_ERR_ARGS
 		fi
+
+		if [ -n "$OCF_RESKEY_xmlfile" ]; then
+			ocf_log err "Cannot use xmlfile if use_virsh is set to 0"
+			return $OCF_ERR_ARGS
+		fi
 	else
-	
+
 		#
-		# If no path is set, use virsh.  Otherwise, use xm.
-		# xm only works with Xen.
+		# Virsh path support.
 		#
-		if [ -z "$OCF_RESKEY_path" ] ||
-		   [ "$OCF_RESKEY_path" = "/etc/xen" ]; then
-			echo "Management tool: virsh"
-			export OCF_RESKEY_use_virsh=1
+		if [ -n "$OCF_RESKEY_path" ] &&
+		   [ "$OCF_RESKEY_path" != "/etc/xen" ]; then
+			if [ -n "$OCF_RESKEY_xmlfile" ]; then
+				ocf_log warning "Using $OCF_RESKEY_xmlfile instead of searching $OCF_RESKEY_path"
+			else
+				search_config_path
+				if [ $? -ne 0 ]; then
+					ocf_log warning "Could not find $OCF_RESKEY_name or $OCF_RESKEY_name.xml in search path $OCF_RESKEY_path"
+					unset OCF_RESKEY_xmlfile
+				else
+					ocf_log debug "Using $OCF_RESKEY_xmlfile"
+				fi
+				choose_management_tool
+			fi
 		else
-			if [ -n "$OCF_RESKEY_use_virsh" ]; then
-				ocf_log warning "Cannot use virsh with 'path' attribute set"
-				ocf_log warning "Setting use_virsh to 0."
-			fi
-
-			if [ "$OCF_RESKEY_hypervisor" != "xen" ]; then
-				ocf_log err "Cannot use $OCF_RESKEY_hypervisor hypervisor with 'path' attribute"
-				return $OCF_ERR_ARGS
-			fi
-
-			echo "Management tool: xm"
-			export OCF_RESKEY_use_virsh=0
+			export OCF_RESKEY_use_virsh=1
 		fi
 	fi
 
 	if [ "$OCF_RESKEY_use_virsh" = "0" ]; then
 
+		echo "Management tool: xm"
 		which xm &> /dev/null
 		if [ $? -ne 0 ]; then
 			ocf_log err "Cannot find 'xm'; is it installed?"
 			return $OCF_ERR_INSTALLED
 		fi
 
-		if [ "$OCF_RESKEY_hypervisor" = "qemu" ] ||
-		   [ "$OCF_RESKEY_hypervisor" = "kvm" ]; then
+		if [ "$OCF_RESKEY_hypervisor" != "xen" ]; then
 			ocf_log err "Cannot use $OCF_RESKEY_hypervisor hypervisor without using virsh"
 			return $OCF_ERR_ARGS
 		fi
 	else
+		echo "Management tool: virsh"
 		which virsh &> /dev/null
 		if [ $? -ne 0 ]; then
 			ocf_log err "Cannot find 'virsh'; is it installed?"
@@ -714,16 +812,10 @@ virsh_migrate()
 		ocf_log err "$err"
 
 		if [ "$err" != "${err/does not exist/}" ]; then
-			return $OCF_NOT_RUNNING
-		fi
-		if [ "$err" != "${err/Domain not found/}" ]; then
-			return $OCF_NOT_RUNNING
-		fi
-		if [ "$err" != "${err/Connection refused/}" ]; then
 			return $OCF_ERR_CONFIGURED
 		fi
-		if [ "$err" != "${err/unable to start guest/}" ]; then
-			return $OCF_NOT_RUNNING
+		if [ "$err" != "${err/Domain not found/}" ]; then
+			return $OCF_ERR_CONFIGURED
 		fi
 
 		return $OCF_ERR_GENERIC
@@ -833,7 +925,21 @@ case $1 in
 	migrate)
 		validate_all || exit $OCF_ERR_ARGS
 		migrate $2 # Send VM to this node
-		exit $?
+		rv=$?
+		if [ $rv -eq $OCF_ERR_GENERIC ]; then
+			# Catch-all: If migration failed with
+			# an unhandled error, do a status check
+			# to see if the VM is really dead.
+			#
+			# If the VM is still in good health, return
+			# a value to rgmanager to indicate the 
+			# non-critical error
+			do_status > /dev/null
+			if [ $? -eq 0 ]; then
+				rv=$OCF_NOT_RUNNING
+			fi
+		fi
+		exit $rv
 		;;
 	reload)
 		exit 0
